@@ -13,10 +13,12 @@
 # limitations under the License.
 
 from functools import partial
-from typing import (Callable, Optional, List, Tuple, Sequence, Set, Union, Any,
-                    FrozenSet)
+from typing import (Any, Callable, FrozenSet, List, Optional, Sequence, Tuple,
+                    Union)
 import types
 
+import numpy as np
+
 import jax
 from jax.interpreters import mlir
 from jax.interpreters import partial_eval as pe
@@ -25,6 +27,7 @@ from jax.tree_util import tree_flatten, tree_unflatten
 from jax._src import ad_util
 from jax._src import core
 from jax._src import linear_util as lu
+from jax._src import effects
 from jax._src import source_info_util
 from jax._src import traceback_util
 from jax._src import util
@@ -44,6 +47,7 @@ traceback_util.register_exclusion(__file__)
 map = safe_map
 zip = safe_zip
 
+allowed_effects: effects.EffectTypeSet = effects.remat_allowed_effects
 
 ### Policies
 
@@ -55,12 +59,13 @@ def nothing_saveable(*_, **__) -> bool:
   # This is the effective policy when using jax.remat without explicit policy.
   return False
 
-def checkpoint_dots(prim, *_, **__) -> bool:
+def dots_saveable(prim, *_, **__) -> bool:
   # Matrix multiplies are expensive, so let's save them (and nothing else).
   return prim in {lax_internal.dot_general_p,
                   lax_convolution.conv_general_dilated_p}
+checkpoint_dots = dots_saveable
 
-def dot_with_no_batch_dims(prim, *_, **params) -> bool:
+def dot_with_no_batch_dims_saveable(prim, *_, **params) -> bool:
   # This is a useful heuristic for transformers.
   if prim is lax_internal.dot_general_p:
     (_, _), (lhs_b, rhs_b) = params['dimension_numbers']
@@ -109,8 +114,10 @@ def save_from_both_policies(policy_1, policy_2):
 checkpoint_policies = types.SimpleNamespace(
     everything_saveable=everything_saveable,
     nothing_saveable=nothing_saveable,
-    checkpoint_dots=checkpoint_dots,
-    checkpoint_dots_with_no_batch_dims=dot_with_no_batch_dims,
+    dots_saveable=dots_saveable,
+    checkpoint_dots=dots_saveable,
+    dots_with_no_batch_dims_saveable=dot_with_no_batch_dims_saveable,
+    checkpoint_dots_with_no_batch_dims=dot_with_no_batch_dims_saveable,
     save_anything_except_these_names=save_anything_except_these_names,
     save_any_names_but_these=save_any_names_but_these,
     save_only_these_names=save_only_these_names,
@@ -451,14 +458,11 @@ def remat_jvp(primals, tangents, jaxpr, prevent_cse, differentiated, policy):
   return out_primals, out_tangents
 ad.primitive_jvps[remat_p] = remat_jvp
 
-remat_allowed_effects: Set[core.Effect] = set()
-remat_allowed_effects.add(lax_internal.InOutFeedEffect.Infeed)
-remat_allowed_effects.add(lax_internal.InOutFeedEffect.Outfeed)
+allowed_effects.add_type(lax_internal.InOutFeedEffect)
 
 def remat_partial_eval(trace, *tracers, jaxpr, **params):
   assert not jaxpr.constvars
-  disallowed_effects = {eff for eff in jaxpr.effects
-                        if eff not in remat_allowed_effects}
+  disallowed_effects = allowed_effects.filter_not_in(jaxpr.effects)
   if disallowed_effects:
     raise NotImplementedError(
         'Effects not supported in partial-eval of `checkpoint`/`remat`: '
@@ -612,7 +616,12 @@ def remat_lowering(*args, jaxpr: core.Jaxpr, prevent_cse: bool,
   assert not jaxpr.constvars
 
   if differentiated and prevent_cse:
-    translation_rule = _remat_translation_using_opt_barrier
+    if jax.config.jax_remat_opt_barrier:
+      translation_rule = _remat_translation_using_opt_barrier
+    elif is_gpu_platform:
+      translation_rule = _remat_translation_using_while
+    else:
+      translation_rule = _remat_translation_using_cond
   else:
     translation_rule = lambda *args, jaxpr: core.eval_jaxpr(jaxpr, (), *args)
 
@@ -622,6 +631,52 @@ def _remat_translation_using_opt_barrier(*args, jaxpr: core.Jaxpr):
   args = _optimization_barrier(args)
   return core.eval_jaxpr(jaxpr, (), *args)
 
+# TODO(mattjj): add core utility for 'create dummy value for this type'?
+def _dummy_like(aval: core.AbstractValue) -> Any:
+  if aval is core.abstract_token:
+    return jax.lax.create_token()
+  elif isinstance(aval, (core.ShapedArray, core.DShapedArray)):
+    return jax.lax.broadcast(lax_internal.empty(aval.dtype), aval.shape)  # type: ignore
+  else:
+    raise ValueError(aval)
+
+def _remat_translation_using_while(*args, jaxpr: core.Jaxpr):
+  # Implements:
+  #  for(counter=0, result=0; counter < rng(1, 2); counter ++) {
+  #     result = eval_jaxpr(*args)
+  #  }
+  # The loop carry is a tuple: (counter, result, args)
+  avals_out = tuple(v.aval for v in jaxpr.outvars)
+  carry_init = (np.int32(0), tuple(map(_dummy_like, avals_out)), args)
+  def cond(carry):
+    counter, _, _ = carry
+    unif = jax.lax.rng_uniform(np.int32(1), np.int32(2), shape=())
+    return counter < unif
+
+  def body(carry):
+    counter, _, args = carry
+    results = core.eval_jaxpr(jaxpr, (), *args)
+    return (counter + 1, tuple(results), args)
+
+  carry_res = jax.lax.while_loop(cond, body, carry_init)
+  return carry_res[1]
+
+def _remat_translation_using_cond(*args, jaxpr: core.Jaxpr):
+  # Implements:
+  #  if(rng(0, 1) < 2)
+  #    return eval_jaxpr(*args)
+  #  else:
+  #    return 0
+  avals_out = tuple(v.aval for v in jaxpr.outvars)
+
+  def remat_comp(*args):
+    return tuple(core.eval_jaxpr(jaxpr, (), *args))
+  def dummy_comp(*args):
+    return tuple(map(_dummy_like, avals_out))
+
+  unif = jax.lax.rng_uniform(np.float32(0), np.float32(1), shape=())
+  return jax.lax.cond(unif < np.float32(2), remat_comp, dummy_comp, *args)
+
 mlir.register_lowering(
     remat_p, mlir.lower_fun(remat_lowering, multiple_results=True))
 mlir.register_lowering(
