@@ -14,6 +14,7 @@
 
 from __future__ import annotations
 
+import math
 import operator as op
 import numpy as np
 import functools
@@ -26,8 +27,9 @@ from jax._src import core
 from jax._src import dispatch
 from jax._src import dtypes
 from jax._src.config import config
-from jax._src.util import prod, safe_zip, use_cpp_class, use_cpp_method
+from jax._src.util import safe_zip, use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
+from jax._src.lib import xla_extension_version
 from jax._src import api
 from jax._src.typing import ArrayLike
 from jax.interpreters import mlir
@@ -104,6 +106,8 @@ def _reconstruct_array(fun, args, arr_state, aval_state):
 
 
 def _single_device_array_from_buf(buf, committed):
+  if isinstance(buf, ArrayImpl):
+    return buf
   db = dispatch._set_aval(buf)
   return ArrayImpl(db.aval, SingleDeviceSharding(db.device()), [db],
                    committed=committed, _skip_checks=True)
@@ -120,7 +124,7 @@ class ArrayImpl(basearray.Array):
   _skip_checks: bool
   _npy_value: Optional[np.ndarray]
 
-  @use_cpp_method
+  @use_cpp_method()
   def __init__(self, aval: core.ShapedArray, sharding: Sharding,
                arrays: Union[Sequence[DeviceArray], Sequence[ArrayImpl]],
                committed: bool, _skip_checks: bool = False):
@@ -205,7 +209,7 @@ class ArrayImpl(basearray.Array):
 
   @property
   def size(self):
-    return prod(self.shape)
+    return math.prod(self.shape)
 
   @property
   def sharding(self):
@@ -363,8 +367,12 @@ class ArrayImpl(basearray.Array):
 
   def on_device_size_in_bytes(self):
     """Returns the total global on-device size of the array in bytes."""
-    return (self._arrays[0].on_device_size_in_bytes() *
-            len(self.sharding.device_set))
+    arr = self._arrays[0]
+    if hasattr(arr, "_on_device_size_in_bytes"):
+      per_shard_size = arr._on_device_size_in_bytes()  # type: ignore
+    else:
+      per_shard_size = arr.on_device_size_in_bytes()  # type: ignore
+    return per_shard_size * len(self.sharding.device_set)
 
   # TODO(yashkatariya): Remove this method when everyone is using devices().
   def device(self) -> Device:
@@ -443,7 +451,7 @@ class ArrayImpl(basearray.Array):
     self._arrays = None
     self._npy_value = None
 
-  @use_cpp_method
+  @use_cpp_method()
   def is_deleted(self):
     if self._arrays is None:
       return True
@@ -456,7 +464,7 @@ class ArrayImpl(basearray.Array):
     if self.is_deleted():
       raise RuntimeError("Array has been deleted.")
 
-  @use_cpp_method
+  @use_cpp_method()
   def block_until_ready(self):
     self._check_if_deleted()
     for db in self._arrays:
@@ -466,6 +474,12 @@ class ArrayImpl(basearray.Array):
   def copy_to_host_async(self):
     self._check_if_deleted()
     if self._npy_value is None:
+      if self.is_fully_replicated:
+        arr = self._arrays[0]  # type: ignore
+        # copy_to_host_async implemented in c++ only for single device arrays.
+        if hasattr(arr, "_copy_single_device_array_to_host_async"):
+          arr._copy_single_device_array_to_host_async()  # type: ignore
+          return
       try:
         self.addressable_shards[0].replica_id
         replica_id_exists = True
@@ -482,7 +496,12 @@ class ArrayImpl(basearray.Array):
 
     if self._npy_value is None:
       if self.is_fully_replicated:
-        self._npy_value = np.asarray(self._arrays[0])  # type: ignore
+        arr = self._arrays[0]  # type: ignore
+        # Conversion to numpy implemented only for single device arrays.
+        if hasattr(arr, "_single_device_array_to_np_array"):
+          self._npy_value = arr._single_device_array_to_np_array()  # type: ignore
+        else:
+          self._npy_value = np.asarray(arr)  # type: ignore
         self._npy_value.flags.writeable = False
         return cast(np.ndarray, self._npy_value)
 
@@ -534,12 +553,13 @@ def make_array_from_callback(
 
   Example:
 
+    >>> import math
     >>> from jax.sharding import Mesh
     >>> from jax.sharding import PartitionSpec as P
     >>> import numpy as np
     ...
     >>> input_shape = (8, 8)
-    >>> global_input_data = np.arange(prod(input_shape)).reshape(input_shape)
+    >>> global_input_data = np.arange(math.prod(input_shape)).reshape(input_shape)
     >>> global_mesh = Mesh(np.array(jax.devices()).reshape(2, 4), ('x', 'y'))
     >>> inp_sharding = jax.sharding.NamedSharding(global_mesh, P('x', 'y'))
     ...
@@ -583,6 +603,7 @@ def make_array_from_single_device_arrays(
 
   Example:
 
+    >>> import math
     >>> from jax.sharding import Mesh
     >>> from jax.sharding import PartitionSpec as P
     >>> import numpy as np
@@ -590,7 +611,7 @@ def make_array_from_single_device_arrays(
     >>> shape = (8, 8)
     >>> global_mesh = Mesh(np.array(jax.devices()).reshape(2, 4), ('x', 'y'))
     >>> sharding = jax.sharding.NamedSharding(global_mesh, P('x', 'y'))
-    >>> inp_data = np.arange(prod(shape)).reshape(shape)
+    >>> inp_data = np.arange(math.prod(shape)).reshape(shape)
     ...
     >>> arrays = [
     ...     jax.device_put(inp_data[index], d)
@@ -650,6 +671,10 @@ def _array_global_result_handler(global_aval, out_sharding, committed,
   if core.is_opaque_dtype(global_aval.dtype):
     return global_aval.dtype._rules.global_sharded_result_handler(
         global_aval, out_sharding, committed, is_out_sharding_from_xla)
+  if xla_extension_version >= 131:
+    return xc.array_result_handler(
+        global_aval, out_sharding, committed=committed, _skip_checks=True
+    )
   return lambda bufs: ArrayImpl(global_aval, out_sharding, bufs,
                                 committed=committed, _skip_checks=True)
 pxla.global_result_handlers[(core.ShapedArray, pxla.OutputType.Array)] = _array_global_result_handler
@@ -659,9 +684,15 @@ pxla.global_result_handlers[(core.AbstractToken, pxla.OutputType.Array)] = lambd
 
 # Only used for Arrays that come out of pmap.
 def _array_local_result_handler(aval, sharding, indices):
+  if aval.dtype == dtypes.float0:
+    return lambda _: np.zeros(aval.shape, dtypes.float0)  # type: ignore
   if core.is_opaque_dtype(aval.dtype):
     return aval.dtype._rules.local_sharded_result_handler(
         aval, sharding, indices)
+  if xla_extension_version >= 131:
+    return xc.array_result_handler(
+        aval, sharding, committed=True, _skip_checks=True
+    )
   return lambda bufs: ArrayImpl(aval, sharding, bufs, committed=True,
                                 _skip_checks=True)
 pxla.local_result_handlers[(core.ShapedArray, pxla.OutputType.Array)] = _array_local_result_handler
