@@ -36,14 +36,14 @@ from jax.tree_util import tree_unflatten
 from jax._src import linear_util as lu
 from jax._src import core
 from jax._src import custom_derivatives
+from jax._src import effects
 from jax._src import prng
 from jax._src import source_info_util
 from jax._src import traceback_util
 from jax._src.config import config
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
-from jax._src.lax import control_flow as cf
-from jax._src.sharding import OpShardingSharding
+from jax._src.sharding import GSPMDSharding
 from jax._src.typing import Array
 from jax._src.util import (as_hashable_function, split_list, safe_map, safe_zip,
                            unzip3, weakref_lru_cache)
@@ -100,20 +100,18 @@ class JaxException(Exception):
 
 @functools.total_ordering
 @dataclasses.dataclass(eq=True, frozen=True)
-class ErrorEffect:
+class ErrorEffect(effects.Effect):
   error_type: Type[JaxException]
   shape_dtypes: Tuple[jax.ShapeDtypeStruct, ...]
 
-  def __post_init__(self):
-    cf.allowed_effects.add(self)
-    mlir.lowerable_effects.add(self)
-
   def __lt__(self, other: 'ErrorEffect'):
     shape_dtypes = lambda x: tuple((sd.shape, str(sd.dtype))  # dtype is not comparable
                                    for sd in x.shape_dtypes)
     unpack = lambda x: (str(x.error_type), shape_dtypes(x))
     return (unpack(self) < unpack(other))
 
+effects.control_flow_allowed_effects.add_type(ErrorEffect)
+effects.lowerable_effects.add_type(ErrorEffect)
 
 class DivisionByZeroError(JaxException):
 
@@ -860,7 +858,7 @@ def pjit_error_check(error, enabled_errors, *vals_in, jaxpr,
   if jax.config.jax_array:
     sharding = pjit._UNSPECIFIED
   else:
-    sharding = OpShardingSharding.get_replicated(
+    sharding = GSPMDSharding.get_replicated(
         list(resource_env.physical_mesh.devices.flat))
 
   new_in_shardings = (*[sharding] * num_error_vals, *in_shardings)
@@ -923,8 +921,8 @@ def custom_jvp_call_rule(in_err, enabled_errors, *in_vals, num_consts,
     return core.eval_jaxpr(jvp_jaxpr, jvp_consts, *primals, *tangents)
 
   jvp, jvp_out_tree = flatten_fun_output(jvp)
-  all_outs = custom_derivatives.custom_jvp_call_p.bind(partial_checkify, jvp,
-                                                       *err_vals, *in_vals)
+  all_outs = custom_derivatives.custom_jvp_call_p.bind(
+      partial_checkify, jvp, *err_vals, *in_vals, **params)
   fst, out_metadata = lu.merge_linear_aux(f_metadata, jvp_out_tree)
   if fst:
     err_and_out_tree, _ = out_metadata
@@ -1127,7 +1125,6 @@ def _check(pred, msg, debug, *fmt_args, **fmt_kwargs):
   _check_error(error, debug=debug)
 
 def _check_error(error, *, debug=False):
-  error = tree_map(core.raise_as_much_as_possible, error)
   if any(map(np.shape, error._pred.values())):
     error = _reduce_any_error(error)
   err_args, tree_def = tree_flatten(error)
