@@ -42,6 +42,7 @@ from jax._src import array
 from jax._src import core
 from jax._src import device_array
 from jax._src import dtypes
+from jax._src import effects
 from jax._src import linear_util as lu
 from jax._src import path
 from jax._src import profiler
@@ -49,19 +50,18 @@ from jax._src import source_info_util
 from jax._src import stages
 from jax._src import traceback_util
 from jax._src import util
+from jax._src import xla_bridge as xb
 from jax._src.abstract_arrays import array_types
 from jax._src.config import config, flags
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
 from jax._src.interpreters import xla
 from jax._src.lib.mlir import ir
-from jax._src.lib.mlir.dialects import use_stablehlo
 from jax._src.lib import pmap_lib
-from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 from jax._src.sharding import (PmapSharding, SingleDeviceSharding,
-                               OpShardingSharding, NamedSharding, PartitionSpec,
-                               Sharding)
+                               GSPMDSharding, NamedSharding, PartitionSpec,
+                               Sharding, XLACompatibleSharding)
 from jax._src.util import flatten, unflatten
 
 
@@ -87,7 +87,6 @@ Backend = xe.Client
 Device = xc.Device
 Buffer = xe.Buffer
 
-XlaLoadedExecutable = xla.XlaLoadedExecutable
 CompileOptions = xc.CompileOptions
 
 map, unsafe_map = util.safe_map, map
@@ -110,7 +109,7 @@ def arg_spec(x: Any) -> ArgSpec:
     if config.jax_array:
       if isinstance(x.sharding, PmapSharding):
         return aval, None
-      return aval, (pjit.to_op_sharding_sharding(x.sharding, x.ndim)  # type: ignore
+      return aval, (pjit.to_gspmd_sharding(x.sharding, x.ndim)  # type: ignore
                     if x._committed else None)
     else:
       return aval, x._device
@@ -322,7 +321,7 @@ def not_none_device_or_backend_on_jit(backend, device, num_ins):
   assert len(da) == 1
   # in_shardings will be marked as replicated regardless of whatever the input
   # had. Given that only a single device is allowed above, this is correct.
-  in_shardings = [OpShardingSharding.get_replicated(da)] * num_ins
+  in_shardings = [GSPMDSharding.get_replicated(da)] * num_ins
   return da, in_shardings
 
 
@@ -500,17 +499,17 @@ def lower_xla_callable(
   # pass long arg lists as tuple for TPU
   tuple_args = should_tuple_args(len(abstract_args), backend.platform)
   axis_env = xla.AxisEnv(nreps, (), ())
-  name_stack = util.new_name_stack(util.wrap_name(name, 'jit'))
+  name_stack = source_info_util.new_name_stack(util.wrap_name(name, 'jit'))
   closed_jaxpr = core.ClosedJaxpr(jaxpr, consts)
   closed_out_type = [
       (a.update(shape=tuple(pe.InDBIdx(d.val - len(consts))
                             if type(d) is pe.InDBIdx else d for d in a.shape))
        if type(a) is core.DShapedArray else a, b) for a, b in out_type]
   module_name = f"jit_{fun.__name__}"
-  unordered_effects = [eff for eff in closed_jaxpr.effects
-                       if eff not in core.ordered_effects]
-  ordered_effects = [eff for eff in closed_jaxpr.effects
-                     if eff in core.ordered_effects]
+  unordered_effects = list(
+    effects.ordered_effects.filter_not_in(closed_jaxpr.effects))
+  ordered_effects = list(
+    effects.ordered_effects.filter_in(closed_jaxpr.effects))
   lowering_result = mlir.lower_jaxpr_to_module(
       module_name, closed_jaxpr, unordered_effects,
       ordered_effects, backend, backend.platform,
@@ -810,7 +809,7 @@ if MYPY:
   ResultHandler = Any
 else:
   class ResultHandler(Protocol):
-    def __call__(self, env: Optional[Sequence[Any]], *args: xla.Buffer) -> Any:
+    def __call__(self, env: Optional[Sequence[Any]], *args: xc.Buffer) -> Any:
       """Boxes raw buffers into their user-facing representation."""
 
 def aval_to_result_handler(sticky_device: Optional[Device],
@@ -853,7 +852,6 @@ def _dynamic_array_result_handler(sticky_device, aval, env, buf):
     return core.DArray(aval.update(shape=tuple(shape)), data)
 
 
-
 result_handlers: Dict[
     Type[core.AbstractValue],
     Callable[[Optional[Device], Any], ResultHandler]] = {}
@@ -894,7 +892,7 @@ def _add_tokens(has_unordered_effects: bool, ordered_effects: List[core.Effect],
   return input_bufs, _remove_tokens
 
 
-def _execute_compiled(name: str, compiled: XlaLoadedExecutable,
+def _execute_compiled(name: str, compiled: xc.LoadedExecutable,
                       input_handler: Optional[Callable],
                       output_buffer_counts: Sequence[int],
                       result_handler: Callable, has_unordered_effects: bool,
@@ -919,7 +917,7 @@ def _execute_compiled(name: str, compiled: XlaLoadedExecutable,
 
 
 def _execute_replicated(name: str,
-                        compiled: XlaLoadedExecutable,
+                        compiled: xc.LoadedExecutable,
                         input_handler: Optional[Callable],
                         output_buffer_counts: Sequence[int],
                         result_handler: Callable,
@@ -939,7 +937,7 @@ def _execute_replicated(name: str,
                 for device in compiled.local_devices()]
   input_bufs_flip = list(unsafe_zip(*input_bufs))
   out_bufs_flat_rep = compiled.execute_sharded_on_local_devices(input_bufs_flip)
-  out_flat = [bufs[0] for bufs in out_bufs_flat_rep]
+  out_flat = [bufs[0] for bufs in out_bufs_flat_rep]  # type: ignore
   check_special(name, out_flat)
   out_bufs = unflatten(out_flat, output_buffer_counts)
   if from_lower_sharding_computation:
@@ -994,20 +992,12 @@ class XlaComputation(stages.XlaLowering):
         use_tuple_args=self.compile_args["tuple_args"])
 
   def mhlo(self) -> ir.Module:
-    if use_stablehlo:
-      return super().mhlo()
-    else:
-      if self.is_trivial():
-        raise ValueError("A trivial computation has no MHLO")
-      return self._hlo
+    return super().mhlo()
 
   def stablehlo(self) -> ir.Module:
-    if use_stablehlo:
-      if self.is_trivial():
-        raise ValueError("A trivial computation has no StableHLO")
-      return self._hlo
-    else:
-      return super().stablehlo()
+    if self.is_trivial():
+      raise ValueError("A trivial computation has no StableHLO")
+    return self._hlo
 
   def compile(self) -> XlaCompiledComputation:
     if self._executable is None:
@@ -1098,8 +1088,8 @@ def compile_or_get_cached(backend, computation: ir.Module, compile_options,
 
 def _cache_read(computation: Union[str, bytes, ir.Module], module_name: str,
                 compile_options: CompileOptions,
-                backend: Backend) -> Optional[XlaLoadedExecutable]:
-  """Looks up `computation` in the persisent compilation cache."""
+                backend: Backend) -> Optional[xc.LoadedExecutable]:
+  """Looks up `computation` in the persistent compilation cache."""
   # Avoid import cycle between jax and jax.experimental
   from jax.experimental.compilation_cache import compilation_cache as cc
 
@@ -1117,7 +1107,7 @@ def _cache_read(computation: Union[str, bytes, ir.Module], module_name: str,
 def _cache_write(serialized_computation: Union[str, bytes, ir.Module],
                  compile_time_secs: float,
                  module_name: str, compile_options: CompileOptions,
-                 backend: Backend, compiled: XlaLoadedExecutable,
+                 backend: Backend, compiled: xc.LoadedExecutable,
                  host_callbacks: List[Any]):
   """Writes `serialized_computation` to the persistent compilation cache."""
   # Avoid import cycle between jax and jax.experimental
@@ -1369,12 +1359,22 @@ def _copy_array_to_device(x: jax.Array, device: Optional[xc.Device]) -> jax.Arra
       committed=(device is not None))
 
 
+# TODO(yashkatariya): Generalize is_compatible_aval (maybe renamed) and use that
+# to check if shardings are compatible with the input.
+def _check_sharding(aval, s):
+  from jax._src import pjit
+
+  if isinstance(s, XLACompatibleSharding) and not isinstance(s, PmapSharding):
+    pjit.pjit_check_aval_sharding(
+        (s,), (aval,), "device_put args", allow_uneven_sharding=False)
+
+  s.shard_shape(aval.shape)  # should raise an Error if incompatible
+
+
 def _device_put_impl(
     x, device: Optional[Union[Device, jax.sharding.Sharding]] = None):
-  from jax.interpreters import pxla
-
   try:
-    a = xla.abstractify(x)
+    aval = xla.abstractify(x)
   except TypeError as err:
     raise TypeError(
         f"Argument '{x}' of type {type(x)} is not a valid JAX type") from err
@@ -1391,13 +1391,15 @@ def _device_put_impl(
       raise ValueError(
           "device_put's second argument must be a Device or a Sharding which "
           f"represents addressable devices, but got {s}")
+
+    _check_sharding(aval, s)
+
     if getattr(x, 'sharding', None) == s:
       return x
     # TODO(mattjj,yashkatariya,phawkins): more runtime fast resharding here?
-    arg_handler = pxla.shard_arg_handlers[type(x)]
-    result_handler = pxla.global_aval_to_result_handler(a, s, True, False)
-    map_ = s.devices_indices_map(x.shape)  # type: ignore
-    return result_handler(arg_handler(x, list(map_), list(map_.values())))
+    result_handler = pxla.global_aval_to_result_handler(aval, s, True, False)
+    map_ = s.devices_indices_map(aval.shape)  # type: ignore
+    return result_handler(pxla.shard_arg(x, list(map_), list(map_.values())))
 
   # Only `Device` exists below. `Sharding` instance is handled above.
   if isinstance(x, array.ArrayImpl):
@@ -1413,7 +1415,7 @@ def _device_put_impl(
   if device_array.type_is_device_array(x):
     return _copy_device_array_to_device(x, device)
 
-  return aval_to_result_handler(device, a)(None, *device_put(x, device))
+  return aval_to_result_handler(device, aval)(None, *device_put(x, device))
 
 
 device_put_p = core.Primitive('device_put')
