@@ -39,9 +39,10 @@ from jax._src import ad_util
 from jax._src import core
 from jax._src import device_array
 from jax._src import dtypes
+from jax._src import effects as effects_lib
 from jax._src import source_info_util
 from jax._src import util
-from jax._src.lib import xla_bridge as xb
+from jax._src import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import hlo
@@ -58,7 +59,7 @@ Value = Any  # = ir.Value
 # mypy implicitly sets this variable to true when type checking.
 MYPY = False
 
-lowerable_effects: Set[core.Effect] = set()
+lowerable_effects: effects_lib.EffectTypeSet = effects_lib.lowerable_effects
 
 
 # IR Helpers
@@ -695,7 +696,8 @@ def lower_jaxpr_to_module(
   if platform in _platforms_with_donation:
     input_output_aliases, donated_args = _set_up_aliases(
         in_avals, out_avals, donated_args)
-  if any(eff not in lowerable_effects for eff in jaxpr.effects):
+  unlowerable_effects = lowerable_effects.filter_not_in(jaxpr.effects)
+  if unlowerable_effects:
     raise ValueError(f'Cannot lower jaxpr with effects: {jaxpr.effects}')
   if any(donated_args):
     # TODO(tomhennigan): At call time we should mark these buffers as deleted.
@@ -731,8 +733,7 @@ def lower_jaxpr_to_module(
     module_name = _module_name_regex.sub("_", module_name)
     ctx.module.operation.attributes["sym_name"] = ir.StringAttr.get(
         module_name)
-    unlowerable_effects = {eff for eff in jaxpr.effects
-                           if eff not in lowerable_effects}
+    unlowerable_effects = lowerable_effects.filter_not_in(jaxpr.effects)
     if unlowerable_effects:
       raise ValueError(
           f'Cannot lower jaxpr with unlowerable effects: {unlowerable_effects}')
@@ -820,11 +821,8 @@ class TokenSet:
   def items(self) -> Sequence[Tuple[core.Effect, Token]]:
     return tuple(self._tokens.items())
 
-  def effects(self) -> Sequence[core.Effect]:
-    return tuple(self._tokens.keys())
-
-  def tokens(self) -> Sequence[Token]:
-    return tuple(self._tokens.values())
+  def effects(self) -> set[core.Effect]:
+    return set(self._tokens.keys())
 
   def subset(self, effects: Sequence[core.Effect]) -> TokenSet:
     """Return a subset of the `TokenSet` restricted to a set of `core.Effect`s."""
@@ -835,10 +833,10 @@ class TokenSet:
     new_tokens = []
     for eff in self.effects():
       if eff in tokens._tokens:
-        new_tokens.append(tokens._tokens[eff])
+        new_tokens.append((eff, tokens._tokens[eff]))
       else:
-        new_tokens.append(self._tokens[eff])
-    return TokenSet(zip(self.effects(), new_tokens))
+        new_tokens.append((eff, self._tokens[eff]))
+    return TokenSet(new_tokens)
 
 def dummy_token_type() -> Sequence[ir.Type]:
   return aval_to_ir_types(core.ShapedArray((0,), np.bool_))
@@ -966,8 +964,7 @@ def lower_jaxpr_to_fun(
     if use_sharding_annotations and ir_arg_shardings is not None:
       for attrs, sharding in zip(arg_attrs, ir_arg_shardings):
         if sharding is not None:
-          attrs["mhlo.sharding"] = ir.StringAttr.get(
-              sharding.SerializeToString())
+          attrs["mhlo.sharding"] = get_sharding_attr(sharding)
 
     if input_output_aliases is not None:
       output_ids = util.unflatten(list(range(len(flat_output_types))),
@@ -988,9 +985,9 @@ def lower_jaxpr_to_fun(
 
   if use_sharding_annotations and ir_result_shardings is not None:
     func_op.result_attrs = ir.ArrayAttr.get([
-        ir.DictAttr.get(
+            ir.DictAttr.get(
             {} if sharding is None else
-            {"mhlo.sharding": ir.StringAttr.get(sharding.SerializeToString())}
+            {"mhlo.sharding": get_sharding_attr(sharding)}
         ) for sharding in ir_result_shardings
     ])
 
@@ -1016,8 +1013,7 @@ def lower_jaxpr_to_fun(
         args.append(hlo.CreateTokenOp().results)
       else:
         args.append(arg)
-    callee_name_stack = xla.extend_name_stack(ctx.name_stack,
-                                              util.wrap_name(name, api_name))
+    callee_name_stack = ctx.name_stack.extend(util.wrap_name(name, api_name))
     out_vals, tokens_out = jaxpr_subcomp(ctx.replace(name_stack=callee_name_stack),
                                          jaxpr.jaxpr, tokens_in, map(ir_constants, jaxpr.consts),
                                          *args, dim_var_values=dim_var_values)
@@ -1028,8 +1024,8 @@ def lower_jaxpr_to_fun(
       for _ in effects:
         outs.append(dummy_token())
     else:
-      for token in tokens_out.tokens():
-        outs.append(token)
+      for eff in effects:
+        outs.append(tokens_out.get(eff))
     for aval, out in zip(jaxpr.out_avals, out_vals):
       if replace_tokens_with_dummy and aval is core.abstract_token:
         outs.append(ir_constants(np.zeros((), np.bool_)))
@@ -1053,7 +1049,8 @@ def _emit_lowering_rule_as_fun(lowering_rule,
 
   input_types = map(aval_to_ir_types, ctx.avals_in)
   output_types = map(aval_to_ir_types, ctx.avals_out)
-  token_types = [token_type() for _ in ctx.tokens_in.items()]
+  effs = list(ctx.tokens_in.effects())
+  token_types = [token_type() for _ in effs]
   input_types = [*dim_var_types, *token_types, *input_types]
   output_types = [*token_types, *output_types]
 
@@ -1070,11 +1067,11 @@ def _emit_lowering_rule_as_fun(lowering_rule,
     unflattened_args = util.unflatten(entry_block.arguments,
                                       map(len, input_types))
     dim_var_values, token_args, unflattened_args = util.split_list(unflattened_args, [num_dim_vars, len(ctx.tokens_in)])
-    sub_ctx = ctx.replace(tokens_in=TokenSet(zip(ctx.tokens_in.effects(), token_args)),
+    sub_ctx = ctx.replace(tokens_in=TokenSet(zip(effs, token_args)),
                           dim_var_values=dim_var_values)
     outs = lowering_rule(sub_ctx, *_unwrap_singleton_ir_values(unflattened_args))
     if sub_ctx.tokens_out:
-      outs = [*sub_ctx.tokens_out.tokens(), outs]
+      outs = [*[sub_ctx.tokens_out.get(eff) for eff in effs], outs]
     func_dialect.ReturnOp(util.flatten(map(wrap_singleton_ir_values, outs)))
   return func_op
 
@@ -1140,7 +1137,7 @@ def jaxpr_subcomp(ctx: ModuleContext, jaxpr: core.Jaxpr,
             f"found for platform {ctx.platform}")
 
       eqn_ctx = ctx.replace(name_stack=source_info.name_stack)
-      effects = [eff for eff in eqn.effects if eff in core.ordered_effects]
+      effects = list(effects_lib.ordered_effects.filter_in(eqn.effects))
       tokens_in = tokens.subset(effects)
       avals_in = map(aval, eqn.invars)
       rule_ctx = LoweringRuleContext(
@@ -1244,12 +1241,13 @@ def _call_lowering(fn_name, stack_name, call_jaxpr, backend, ctx, avals_in,
   if isinstance(call_jaxpr, core.Jaxpr):
     call_jaxpr = core.ClosedJaxpr(call_jaxpr, ())
   xla.check_backend_matches(backend, ctx.platform)
-  effects = tokens_in.effects()
+  effects = list(tokens_in.effects())
   output_types = map(aval_to_ir_types, avals_out)
   output_types = [token_type()] * len(effects) + output_types
   flat_output_types = util.flatten(output_types)
   symbol_name = _lower_jaxpr_to_fun_cached(ctx, fn_name, call_jaxpr, effects).name.value
-  args = tuple([*dim_var_values, *tokens_in.tokens(), *args])
+  tokens = [tokens_in.get(eff) for eff in effects]
+  args = tuple([*dim_var_values, *tokens, *args])
   call = func_dialect.CallOp(flat_output_types,
                              ir.FlatSymbolRefAttr.get(symbol_name),
                              flatten_lowering_ir_args(args))
@@ -1491,8 +1489,7 @@ def _wrap_with_spmd_op(name: str,
                         called_computations=ir.ArrayAttr.get([]),
                         operand_layouts=None,
                         result_layouts=None)
-  op.attributes["mhlo.sharding"] = ir.StringAttr.get(
-      sharding_proto.SerializeToString())
+  set_sharding(op, sharding_proto)
   return op.result
 
 def wrap_with_sharding_op(x: ir.Value,
@@ -1505,8 +1502,15 @@ wrap_with_full_to_shard_op = partial(_wrap_with_spmd_op, "SPMDFullToShardShape")
 wrap_with_shard_to_full_op = partial(_wrap_with_spmd_op, "SPMDShardToFullShape")
 
 def set_sharding(op, sharding_proto: xc.OpSharding):
-  op.attributes["mhlo.sharding"] = ir.StringAttr.get(
-      sharding_proto.SerializeToString())
+  op.attributes["mhlo.sharding"] = get_sharding_attr(sharding_proto)
+
+
+def get_sharding_attr(sharding_proto: xc.OpSharding):
+  if xc.mlir_api_version >= 44:
+    return ir.StringAttr.get(repr(xc.HloSharding.from_proto(sharding_proto)))
+  else:
+    return ir.StringAttr.get(sharding_proto.SerializeToString())
+
 
 # MLIR lowerings for lax primitives
 
@@ -1544,7 +1548,6 @@ def cache_lowering(f):
   return cached_lowering
 
 
-
 def xla_computation_to_mlir_module(xla_computation: xc.XlaComputation
                                   ) -> ir.Module:
   module_str = xc._xla.mlir.xla_computation_to_mlir_module(xla_computation)
