@@ -31,6 +31,7 @@ from jax.tree_util import tree_flatten, tree_unflatten
 from jax._src import ad_util
 from jax._src import core
 from jax._src import dtypes
+from jax._src import effects
 from jax._src import linear_util as lu
 from jax._src import source_info_util
 from jax._src import util
@@ -38,8 +39,7 @@ from jax._src import state
 from jax._src.core import ConcreteArray, raise_to_shaped, replace_jaxpr_effects
 from jax._src.lax import lax
 from jax._src.traceback_util import api_boundary
-from jax._src.util import (safe_map, extend_name_stack, split_list,
-                           partition_list)
+from jax._src.util import (safe_map, split_list, partition_list)
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import hlo
 import numpy as np
@@ -136,7 +136,7 @@ def switch(index, branches: Sequence[Callable], *operands,
                           out_trees[0], jaxprs[0].out_avals,
                           out_tree, jaxpr.out_avals)
   joined_effects = core.join_effects(*(jaxpr.effects for jaxpr in jaxprs))
-  disallowed_effects = joined_effects - allowed_effects
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `switch`: {disallowed_effects}')
@@ -236,7 +236,7 @@ def _cond(pred, true_fun: Callable, false_fun: Callable, *operands,
 
   jaxprs, consts, out_trees = _initial_style_jaxprs_with_common_consts(
       (true_fun, false_fun), ops_tree, ops_avals, 'cond')
-  if any(isinstance(op_aval, state.ShapedArrayRef) for op_aval in ops_avals):
+  if any(isinstance(op_aval, state.AbstractRef) for op_aval in ops_avals):
     raise ValueError("Cannot pass `Ref`s into `cond`.")
   true_jaxpr, false_jaxpr = jaxprs
   out_tree, false_out_tree = out_trees
@@ -245,7 +245,7 @@ def _cond(pred, true_fun: Callable, false_fun: Callable, *operands,
                         out_tree, true_jaxpr.out_avals,
                         false_out_tree, false_jaxpr.out_avals)
   joined_effects = core.join_effects(true_jaxpr.effects, false_jaxpr.effects)
-  disallowed_effects = joined_effects - allowed_effects
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `cond`: {disallowed_effects}')
@@ -302,23 +302,23 @@ def _cond_with_per_branch_args(pred,
                lambda op: false_fun(op[1]),
                (true_operand, false_operand))
 
+def _join_cond_effects(branches: Sequence[core.Jaxpr]) -> effects.Effects:
+  joined_effects = set()
+  for b in branches:
+    for eff in b.effects:
+      if isinstance(eff, effects.JaxprInputEffect):
+        # Offset index to handle predicate
+        eff = eff.replace(input_index=eff.input_index + 1)
+      joined_effects.add(eff)
+  return joined_effects
+
 def _cond_abstract_eval(*avals, branches, **_):
-  joined_effects = core.join_effects(*(b.effects for b in branches))
-  disallowed_effects = joined_effects - allowed_effects
+  joined_effects = _join_cond_effects(branches)
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `cond`: {disallowed_effects}')
-  joined_effects = core.join_effects(*(b.effects for b in branches))
-  state_effects = {eff for eff in joined_effects if isinstance(eff,
-    state.RefEffect)}
-  jaxpr_aval_effects = state.get_ref_state_effects(
-      [v.aval for v in branches[0].jaxpr.invars], joined_effects)
-  aval_effects = [set(eff.replace(ref_aval=aval) for eff in effs) for aval, effs
-      in zip(avals[1:], jaxpr_aval_effects)
-      if isinstance(aval, state.ShapedArrayRef)]
-  nonlocal_state_effects = core.join_effects(*aval_effects)
-  all_effects = (joined_effects - state_effects) | nonlocal_state_effects
-  return map(raise_to_shaped, branches[0].out_avals), all_effects
+  return map(raise_to_shaped, branches[0].out_avals), joined_effects
 
 def _bcast_select(pred, on_true, on_false):
   if np.ndim(pred) != np.ndim(on_true):
@@ -547,7 +547,7 @@ def _cond_partial_eval_custom(saveable, unks_in, inst_in, eqn):
   out_binders_known, _ = partition_list(unks_out, eqn.outvars)
   linear_known = [l for l, uk in zip(eqn.params['linear'], ops_uk) if not uk]
   params_known = dict(branches=branches_known, linear=tuple(linear_known))
-  effects_known = core.join_effects(*(b.effects for b in branches_known))
+  effects_known = _join_cond_effects(branches_known)
   eqn_known = pe.new_jaxpr_eqn(
       ins_known, [*out_binders_known, *res_binders], cond_p, params_known,
       effects_known, eqn.source_info)
@@ -556,7 +556,7 @@ def _cond_partial_eval_custom(saveable, unks_in, inst_in, eqn):
   _, out_binders_staged = partition_list(inst_out, eqn.outvars)
   linear_staged = [False] * len(res_binders) + list(eqn.params['linear'])
   params_staged = dict(branches=branches_staged, linear=tuple(linear_staged))
-  effects_staged = core.join_effects(*(b.effects for b in branches_staged))
+  effects_staged = _join_cond_effects(branches_staged)
   eqn_staged = pe.new_jaxpr_eqn(
       [eqn.invars[0], *res_binders, *eqn.invars[1:]], out_binders_staged,
       cond_p, params_staged, effects_staged, eqn.source_info)
@@ -668,6 +668,7 @@ def _cond_dce_rule(used_outputs: List[bool], eqn: core.JaxprEqn,
   new_params = dict(eqn.params, branches=tuple(dce_branches),
                     linear=tuple(dce_linear))
   new_effects = core.join_effects(*(b.effects for b in dce_branches))
+  new_effects = _join_cond_effects(dce_branches_)
   new_eqn = pe.new_jaxpr_eqn(
       [v for v, used in zip(eqn.invars, [True, *used_inputs]) if used],
       [v for v, used in zip(eqn.outvars, used_outputs) if used],
@@ -750,8 +751,8 @@ def _cond_typecheck(*in_atoms, branches, linear):
   jaxpr0 = branches[0]
   jaxpr0_in_avals_str = _avals_short(jaxpr0.in_avals)
   jaxpr0_out_avals_str = _avals_short(jaxpr0.out_avals)
-  joined_effects = core.join_effects(*(b.effects for b in branches))
-  disallowed_effects = joined_effects - allowed_effects
+  joined_effects = _join_cond_effects(branches)
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `cond`: {disallowed_effects}')
@@ -787,7 +788,6 @@ def _cond_typecheck(*in_atoms, branches, linear):
     raise core.JaxprTypeError(
       f'cond branches take input types {jaxpr0_in_avals_str}, '
       f'called with operands of type {_avals_short(op_avals)}')
-  joined_effects = core.join_effects(*(b.effects for b in branches))
   return jaxpr0.out_avals, joined_effects
 
 def cond_bind(*args, branches, linear):
@@ -818,8 +818,7 @@ pe.dce_rules[cond_p] = _cond_dce_rule
 def _cond_lowering(ctx, index, *args, branches, linear):
   del linear  # Unused.
   joined_effects = core.join_effects(*(branch.effects for branch in branches))
-  ordered_effects = [eff for eff in joined_effects
-                     if eff in core.ordered_effects]
+  ordered_effects = list(effects.ordered_effects.filter_in(joined_effects))
   num_tokens = len(ordered_effects)
   tokens_in = ctx.tokens_in.subset(ordered_effects)
   output_token_types = [mlir.token_type() for _ in ordered_effects]
@@ -832,12 +831,12 @@ def _cond_lowering(ctx, index, *args, branches, linear):
   # captures.
   case_op = hlo.CaseOp(flat_output_types, index=index,
                        num_branches=len(branches))
-  name_stack = extend_name_stack(ctx.module_context.name_stack, 'cond')
+  name_stack = ctx.module_context.name_stack.extend('cond')
   for i, jaxpr in enumerate(branches):
     branch = case_op.regions[i].blocks.append()
     with ir.InsertionPoint(branch):
       sub_ctx = ctx.module_context.replace(
-          name_stack=xla.extend_name_stack(name_stack, f'branch_{i}_fun'))
+          name_stack=name_stack.extend(f'branch_{i}_fun'))
       out_vals, tokens_out = mlir.jaxpr_subcomp(
           sub_ctx, jaxpr.jaxpr, tokens_in,
           map(mlir.ir_constants, jaxpr.consts),
@@ -866,5 +865,5 @@ def _cond_state_discharge_rule(in_avals, out_avals, *args, branches, linear):
   new_invals = []
   for aval in in_avals:
     new_invals.append(
-        next(ref_val_iter) if isinstance(aval, state.ShapedArrayRef) else None)
+        next(ref_val_iter) if isinstance(aval, state.AbstractRef) else None)
   return new_invals, out_vals
