@@ -35,6 +35,7 @@ from jax._src import ad_checkpoint
 from jax._src import ad_util
 from jax._src import api
 from jax._src import dtypes
+from jax._src import effects
 from jax._src import source_info_util
 from jax._src import util
 from jax._src.lax import lax
@@ -45,7 +46,6 @@ from jax._src.lib.mlir.dialects import hlo
 from jax._src.numpy.ufuncs import logaddexp
 from jax._src.traceback_util import api_boundary
 from jax._src.util import (
-    extend_name_stack,
     partition_list,
     safe_map,
     safe_zip,
@@ -266,7 +266,7 @@ def scan(f: Callable[[Carry, X], Tuple[Carry, Y]],
                         # Extract the subtree and avals for the first element of the return tuple
                         out_tree_children[0], carry_avals_out,
                         init_tree, carry_avals)
-  disallowed_effects = jaxpr.effects - allowed_effects
+  disallowed_effects = allowed_effects.filter_not_in(jaxpr.effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `scan`: {disallowed_effects}')
@@ -1098,7 +1098,7 @@ def while_loop(cond_fun: Callable[[T], BooleanNumeric],
                         body_tree, body_jaxpr.out_avals,
                         in_tree_children[0], init_avals)
   effects = core.join_effects(cond_jaxpr.effects, body_jaxpr.effects)
-  disallowed_effects = effects - allowed_effects
+  disallowed_effects = allowed_effects.filter_not_in(effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `while`: {disallowed_effects}')
@@ -1107,10 +1107,29 @@ def while_loop(cond_fun: Callable[[T], BooleanNumeric],
                       body_nconsts=len(body_consts), body_jaxpr=body_jaxpr)
   return tree_unflatten(body_tree, outs)
 
-def _while_loop_abstract_eval(*args, cond_jaxpr, body_jaxpr, **kwargs):
-  del args, kwargs
-  joined_effects = core.join_effects(cond_jaxpr.effects, body_jaxpr.effects)
-  disallowed_effects = joined_effects - allowed_effects
+def _join_while_effects(body_jaxpr, cond_jaxpr, body_nconsts, cond_nconsts
+                       ) -> effects.Effects:
+  joined_effects = set()
+  for eff in cond_jaxpr.effects:
+    if isinstance(eff, effects.JaxprInputEffect):
+      index = eff.input_index
+      if index >= cond_nconsts:
+        index += body_nconsts
+      eff = eff.replace(input_index=index)
+    joined_effects.add(eff)
+  for eff in body_jaxpr.effects:
+    if isinstance(eff, effects.JaxprInputEffect):
+      index = eff.input_index + cond_nconsts
+      eff = eff.replace(input_index=index)
+    joined_effects.add(eff)
+  return joined_effects
+
+def _while_loop_abstract_eval(*avals, cond_jaxpr, body_jaxpr, body_nconsts,
+                              cond_nconsts):
+  del avals
+  joined_effects = _join_while_effects(body_jaxpr, cond_jaxpr, body_nconsts,
+                                       cond_nconsts)
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
   if disallowed_effects:
     raise NotImplementedError(
         f'Effects not supported in `while`: {disallowed_effects}')
@@ -1422,8 +1441,7 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
                     body_nconsts):
   pred_aval = cond_jaxpr.out_avals[0]
   batched = bool(pred_aval.shape)
-  cond_ordered_effects = [eff for eff in cond_jaxpr.effects if eff in
-                          core.ordered_effects]
+  cond_ordered_effects = effects.ordered_effects.filter_in(cond_jaxpr.effects)
   if cond_ordered_effects:
     def cond(args):
       # Pred can be batched
@@ -1448,8 +1466,7 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
     return mlir.lower_fun(fun)(ctx, *args)
 
   loop_carry_types = _map(mlir.aval_to_ir_types, ctx.avals_in)
-  body_effects = [eff for eff in body_jaxpr.effects
-                  if eff in core.ordered_effects]
+  body_effects = effects.ordered_effects.filter_in(body_jaxpr.effects)
   num_tokens = len(body_effects)
   tokens = [ctx.tokens_in.get(eff) for eff in body_effects]
   token_types = [mlir.token_type() for _ in tokens]
@@ -1462,7 +1479,7 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
 
   # Loop condition
   cond_block = while_op.regions[0].blocks.append(*flat_loop_carry_types)
-  name_stack = extend_name_stack(ctx.module_context.name_stack, 'while')
+  name_stack = ctx.module_context.name_stack.extend('while')
   with ir.InsertionPoint(cond_block):
     flat_cond_args = [
         cond_block.arguments[i] for i in range(len(flat_loop_carry_types))
@@ -1471,8 +1488,7 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
     # Remove tokens from cond args
     cond_args = cond_args[num_tokens:]
     x, _, z = util.split_list(cond_args, [cond_nconsts, body_nconsts])
-    cond_ctx = ctx.module_context.replace(
-        name_stack=xla.extend_name_stack(name_stack, 'cond'))
+    cond_ctx = ctx.module_context.replace(name_stack=name_stack.extend('cond'))
     ((pred,),), _ = mlir.jaxpr_subcomp(cond_ctx, cond_jaxpr.jaxpr, mlir.TokenSet(),
                                        _map(mlir.ir_constants, cond_jaxpr.consts),
                                        *(x + z), dim_var_values=ctx.dim_var_values)
@@ -1503,16 +1519,14 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
     token_args, body_args = util.split_list(body_args, [num_tokens])
     tokens_in = mlir.TokenSet(zip(body_effects, token_args))
     x, y, z = util.split_list(body_args, [cond_nconsts, body_nconsts])
-    body_ctx = ctx.module_context.replace(
-        name_stack=xla.extend_name_stack(name_stack, 'body'))
+    body_ctx = ctx.module_context.replace(name_stack=name_stack.extend('body'))
     new_z, tokens_out = mlir.jaxpr_subcomp(body_ctx, body_jaxpr.jaxpr,
         tokens_in, _map(mlir.ir_constants, body_jaxpr.consts),
         *(y + z), dim_var_values=ctx.dim_var_values)
     out_tokens = [tokens_out.get(eff) for eff in body_effects]
     if batched:
       body_pred_ctx = ctx.module_context.replace(
-          name_stack=xla.extend_name_stack(name_stack,
-                                           'body_pred'))
+          name_stack=name_stack.extend('body_pred'))
       ((body_pred,),), _ = mlir.jaxpr_subcomp(
           body_pred_ctx, cond_jaxpr.jaxpr, mlir.TokenSet(),
           _map(mlir.ir_constants, cond_jaxpr.consts),
@@ -1533,10 +1547,12 @@ def _while_lowering(ctx, *args, cond_jaxpr, body_jaxpr, cond_nconsts,
 def _while_typecheck(*in_atoms, cond_jaxpr, body_jaxpr, cond_nconsts,
                      body_nconsts):
   # TODO(frostig,mattjj): check cond_jaxpr, body_jaxpr types
-  joined_effects = core.join_effects(cond_jaxpr.effects, body_jaxpr.effects)
-  if joined_effects - allowed_effects:
+  joined_effects = _join_while_effects(body_jaxpr, cond_jaxpr, body_nconsts,
+                                       cond_nconsts)
+  disallowed_effects = allowed_effects.filter_not_in(joined_effects)
+  if disallowed_effects:
     raise NotImplementedError(
-        f'Effects not supported in `while`: {joined_effects - allowed_effects}')
+        f'Effects not supported in `while`: {disallowed_effects}')
   return body_jaxpr.out_avals, joined_effects
 
 while_p = core.AxisPrimitive('while')
