@@ -24,8 +24,4 @@ from jax.lib import xla_client
 import jaxlib.mlir.dialects.stablehlo as stablehlo
 
 # Alias that is set up to abstract away the transition from MHLO to StableHLO.
-use_stablehlo = xla_client.mlir_api_version >= 42
-if use_stablehlo:
-  import jaxlib.mlir.dialects.stablehlo as hlo
-else:
-  import jaxlib.mlir.dialects.mhlo as hlo  # type: ignore[no-redef]
+import jaxlib.mlir.dialects.stablehlo as hlo
