@@ -25,6 +25,7 @@ from jax._src import core
 from jax._src import linear_util as lu
 from jax import stages
 from jax._src import dispatch
+from jax._src import effects
 from jax.tree_util import (tree_flatten, tree_unflatten, all_leaves, tree_map,
                            treedef_tuple)
 from jax._src.api_util import (flatten_fun_nokwargs, flatten_axes,
@@ -35,12 +36,12 @@ from jax._src import traceback_util
 from jax._src.config import config
 from jax.errors import JAXTypeError
 from jax._src.array import ArrayImpl
-from jax.experimental.global_device_array import GlobalDeviceArray
+from jax._src.global_device_array import GlobalDeviceArray
 from jax._src.sharding import NamedSharding
 from jax._src.interpreters import mlir
 from jax.interpreters import partial_eval as pe
 from jax._src.interpreters import pxla
-from jax.interpreters import xla
+from jax._src.interpreters import xla
 from jax.interpreters import batching
 from jax.interpreters import ad
 from jax._src.util import (safe_map, safe_zip, HashableFunction, unzip2, unzip3,
@@ -397,7 +398,7 @@ def xmap(fun: Callable,
   Note that the contraction in the program is performed over the positional axes,
   while named axes are just a convenient way to achieve batching. While this
   might seem like a silly example at first, it might turn out to be useful in
-  practice, since with conjuction with ``axis_resources`` this makes it possible
+  practice, since with conjunction with ``axis_resources`` this makes it possible
   to implement a distributed matrix-multiplication in just a few lines of code::
 
     devices = np.array(jax.devices())[:4].reshape((2, 2))
@@ -1373,9 +1374,9 @@ def _xmap_lowering_rule_replica(ctx, *in_nodes,
   # We in-line here rather than generating a Call HLO as in the xla_call
   # translation rule just because the extra tuple stuff is a pain.
   sub_ctx = ctx.module_context.replace(
-      name_stack=xla.extend_name_stack(ctx.module_context.name_stack,
-                                       wrap_name(name, 'xmap')))
-  if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
+      name_stack=ctx.module_context.name_stack.extend(wrap_name(name, 'xmap')))
+  if any(effects.ordered_effects.contains(eff) for eff
+         in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   tiled_outs, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr, mlir.TokenSet(),
                                      const_nodes, *tiled_ins,
@@ -1440,9 +1441,9 @@ def _xmap_lowering_rule_spmd(ctx, *global_in_nodes,
   # We in-line here rather than generating a Call HLO as in the xla_call
   # translation rule just because the extra tuple stuff is a pain.
   sub_ctx = ctx.module_context.replace(
-      name_stack=xla.extend_name_stack(ctx.module_context.name_stack,
-                                       wrap_name(name, 'xmap')))
-  if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
+      name_stack=ctx.module_context.name_stack.extend(wrap_name(name, 'xmap')))
+  if any(effects.ordered_effects.contains(eff) for eff
+         in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   global_out_nodes, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr,
       mlir.TokenSet(), const_nodes, *sharded_global_in_nodes,
@@ -1491,10 +1492,10 @@ def _xmap_lowering_rule_spmd_manual(ctx, *global_in_nodes,
   # translation rule just because the extra tuple stuff is a pain.
   assert isinstance(ctx.module_context.axis_context, mlir.SPMDAxisContext)
   sub_ctx = ctx.module_context.replace(
-      name_stack=xla.extend_name_stack(ctx.module_context.name_stack,
-                                       wrap_name(name, 'xmap')),
+      name_stack=ctx.module_context.name_stack.extend(wrap_name(name, 'xmap')),
       axis_context=ctx.module_context.axis_context.extend_manual(manual_mesh_axes))
-  if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
+  if any(effects.ordered_effects.contains(eff) for eff
+         in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   global_out_nodes, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr,
       mlir.TokenSet(), const_nodes, *([n] for n in global_in_nodes),
@@ -1860,7 +1861,7 @@ def _check_no_loop_collectives(jaxpr, loop_axis_resources):
 def _fix_inferred_spmd_sharding(jaxpr, resource_env, gen_fresh_name = None):
   from jax._src.pjit import (
       sharding_constraint_p, ParsedPartitionSpec, get_unconstrained_dims,
-      OpShardingSharding)
+      GSPMDSharding)
 
   rec = lambda jaxpr: _fix_inferred_spmd_sharding(jaxpr, resource_env, gen_fresh_name)
   if isinstance(jaxpr, core.ClosedJaxpr):
@@ -1878,12 +1879,11 @@ def _fix_inferred_spmd_sharding(jaxpr, resource_env, gen_fresh_name = None):
       mps = NamedSharding._from_parsed_pspec(
           resource_env.physical_mesh, ParsedPartitionSpec((), ()))
       unconstrained_dims = get_unconstrained_dims(mps)
-      op_sharding_sharding = OpShardingSharding.get_replicated(
-          mps._device_assignment)
+      gspmd_sharding = GSPMDSharding.get_replicated(mps._device_assignment)
       new_eqns.append(core.JaxprEqn(
           [tmpvar], [outvar], sharding_constraint_p,
           dict(resource_env=resource_env,
-               sharding=op_sharding_sharding,
+               sharding=gspmd_sharding,
                unconstrained_dims=unconstrained_dims),
           set(),
           eqn.source_info))
