@@ -14,6 +14,7 @@
 
 import builtins
 from functools import partial
+import math
 import operator
 from typing import overload, Any, Callable, Literal, Optional, Sequence, Tuple, Union
 import warnings
@@ -24,14 +25,13 @@ from jax import lax
 from jax._src import api
 from jax._src import core
 from jax._src import dtypes
-from jax._src.numpy.ndarray import ndarray
 from jax._src.numpy.util import (
     _broadcast_to, _check_arraylike, _complex_elem_type,
     _promote_dtypes_inexact, _promote_dtypes_numeric, _where, _wraps)
 from jax._src.lax import lax as lax_internal
 from jax._src.typing import Array, ArrayLike, DType, DTypeLike
 from jax._src.util import (
-    canonicalize_axis as _canonicalize_axis, maybe_named_axis, prod as _prod)
+    canonicalize_axis as _canonicalize_axis, maybe_named_axis)
 
 
 _all = builtins.all
@@ -43,7 +43,7 @@ Axis = Union[None, int, Sequence[int]]
 
 def _asarray(a: ArrayLike) -> Array:
   # simplified version of jnp.asarray() for local use.
-  return a if isinstance(a, ndarray) else api.device_put(a)
+  return a if isinstance(a, Array) else api.device_put(a)
 
 def _isscalar(element: Any) -> bool:
   if hasattr(element, '__jax_array__'):
@@ -91,7 +91,7 @@ def _reduction(a: ArrayLike, name: str, np_fun: Any, op: ReductionOp, init_val:
     raise ValueError(f"reduction operation {name} does not have an identity, so to use a "
                      f"where mask one has to specify 'initial'")
 
-  a = a if isinstance(a, ndarray) else _asarray(a)
+  a = a if isinstance(a, Array) else _asarray(a)
   a = preproc(a) if preproc else a
   pos_dims, dims = _reduction_dims(a, axis)
 
@@ -371,7 +371,7 @@ def _average(a: ArrayLike, axis: Axis = None, weights: Optional[ArrayLike] = Non
     if axis is None:
       weights_sum = lax.full((), core.dimension_as_value(a.size), dtype=avg.dtype)
     elif isinstance(axis, tuple):
-      weights_sum = lax.full_like(avg, _prod(core.dimension_as_value(a.shape[d]) for d in axis))
+      weights_sum = lax.full_like(avg, math.prod(core.dimension_as_value(a.shape[d]) for d in axis))
     else:
       weights_sum = lax.full_like(avg, core.dimension_as_value(a.shape[axis]))  # type: ignore[index]
   else:
