@@ -24,6 +24,7 @@ import jax
 from jax._src import core
 from jax._src.util import safe_map, safe_zip, use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
+from jax._src.lib import xla_extension_version
 from jax.interpreters import mlir
 from jax._src.interpreters import pxla
 
@@ -75,10 +76,10 @@ class Sharding(metaclass=abc.ABCMeta):
     """Returns True if two shardings put the same logical array
     (sharded/unsharded) on the same device(s).
 
-    For example, every XLACompatibleSharding lowers to OpShardingSharding which
+    For example, every XLACompatibleSharding lowers to GSPMDSharding which
     is a general representation. So `jax.sharding.NamedSharding` is equivalent
     to `jax.sharding.PositionalSharding` if both of them lower to the same
-    OpShardingSharding.
+    GSPMDSharding.
     """
     raise NotImplementedError('Subclasses should implement this method.')
 
@@ -136,9 +137,8 @@ class XLACompatibleSharding(Sharding, metaclass=abc.ABCMeta):
   @functools.lru_cache(maxsize=4096)
   def devices_indices_map(self, global_shape: Shape) -> Mapping[Device, Index]:
     op_sharding = self._to_xla_op_sharding(len(global_shape))
-    op_sharding_sharding = OpShardingSharding(self._device_assignment,
-                                              op_sharding)
-    return op_sharding_sharding.devices_indices_map(global_shape)
+    gspmd_sharding = GSPMDSharding(self._device_assignment, op_sharding)
+    return gspmd_sharding.devices_indices_map(global_shape)
 
   @functools.cached_property
   def _addressable_device_assignment(self) -> XLADeviceAssignment:
@@ -286,7 +286,7 @@ class NamedSharding(XLACompatibleSharding):
     >>> named_sharding = jax.sharding.NamedSharding(mesh, spec)
   """
 
-  @use_cpp_method
+  @use_cpp_method()
   def __init__(
       self, mesh: pxla.Mesh, spec: PartitionSpec, _parsed_pspec = None):
 
@@ -397,7 +397,7 @@ class SingleDeviceSharding(XLACompatibleSharding):
     ...     jax.devices()[0])
   """
 
-  @use_cpp_method
+  @use_cpp_method()
   def __init__(self, device: Device):
     self._device = device
 
@@ -435,7 +435,7 @@ class SingleDeviceSharding(XLACompatibleSharding):
 @use_cpp_class(xc.PmapSharding)
 class PmapSharding(XLACompatibleSharding):
 
-  @use_cpp_method
+  @use_cpp_method()
   def __init__(self, devices: np.ndarray, sharding_spec: pxla.ShardingSpec):
     self.devices = devices
     # The sharding spec should be pmap's sharding spec.
@@ -518,12 +518,20 @@ class PmapSharding(XLACompatibleSharding):
   @functools.lru_cache(maxsize=4096)
   def shard_shape(self, global_shape: Shape) -> Shape:
     sharded_dim = None
+    sharded_dim_size = None
     for i, s in enumerate(self.sharding_spec.sharding):
       if isinstance(s, pxla.Unstacked):
         sharded_dim = i
+        sharded_dim_size = s.size
         break
     if sharded_dim is None:
       return global_shape
+    if global_shape[sharded_dim] != sharded_dim_size:
+      raise ValueError(
+          f'The sharded dimension must be equal to the number of '
+          f'devices passed to PmapSharding. Got sharded dimension {sharded_dim} '
+          f'with value {global_shape[sharded_dim]} in shape {global_shape} and '
+          f'the number of devices={len(self._device_assignment)}')
     return global_shape[:sharded_dim] + global_shape[sharded_dim+1:]
 
 
@@ -645,10 +653,10 @@ class DeviceIdSet:
             self._ids == other._ids)
 
 
-@use_cpp_class(xc.OpShardingSharding)
-class OpShardingSharding(XLACompatibleSharding):
+@use_cpp_class(xc.GSPMDSharding if xla_extension_version >= 129 else xc.OpShardingSharding)  # type: ignore
+class GSPMDSharding(XLACompatibleSharding):
 
-  @use_cpp_method
+  @use_cpp_method()
   def __init__(self, devices: Sequence[Device], op_sharding: xc.OpSharding):
     self._devices = tuple(devices)
     self._op_sharding = op_sharding
@@ -661,7 +669,7 @@ class OpShardingSharding(XLACompatibleSharding):
     return hash(xc.HloSharding.from_proto(self._op_sharding))
 
   def __eq__(self, other):
-    if not isinstance(other, OpShardingSharding):
+    if not isinstance(other, GSPMDSharding):
       return False
     if id(self) == id(other):
       return True
@@ -674,7 +682,7 @@ class OpShardingSharding(XLACompatibleSharding):
     return self._hash
 
   def __repr__(self):
-    return f'OpShardingSharding({repr(xc.HloSharding.from_proto(self._op_sharding))})'
+    return f'GSPMDSharding({repr(xc.HloSharding.from_proto(self._op_sharding))})'
 
   def is_compatible_aval(self, aval_shape: Shape):
     num_ways_dim_sharded, _ = pxla.get_num_ways_dim_sharded(self._op_sharding)
@@ -705,3 +713,8 @@ class OpShardingSharding(XLACompatibleSharding):
   def get_replicated(cls, device_assignment):
     proto = _get_replicated_op_sharding()
     return cls(device_assignment, proto)
+
+
+# TODO(yashkatariya); Remove OpShardingSharding after 3 months from Feb 17, 2023
+# per the deprecation policy.
+OpShardingSharding = GSPMDSharding
