@@ -13,7 +13,7 @@
 # limitations under the License.
 
 import functools
-from functools import partial, cached_property
+from functools import partial
 import itertools as it
 import logging
 import operator
@@ -269,12 +269,6 @@ def weakref_lru_cache(call: Callable, maxsize=2048):
   """
   return xc.weakref_lru_cache(config._trace_context, call, maxsize)
 
-def prod(xs):
-  out = 1
-  for x in xs:
-    out *= x
-  return out
-
 class Unhashable:
   __slots__ = ["val"]
 
@@ -330,18 +324,6 @@ def get_module_functions(module):
 def wrap_name(name, transform_name):
   return transform_name + '(' + name + ')'
 
-def new_name_stack(name: str = ''):
-  from jax._src import source_info_util
-  name_stack = source_info_util.NameStack()
-  if name:
-    name_stack = name_stack.extend(name)
-  return name_stack
-
-def extend_name_stack(stack, name: str):
-  from jax._src import source_info_util
-  assert isinstance(stack, source_info_util.NameStack), stack
-  return stack.extend(name)
-
 def canonicalize_axis(axis, num_dims) -> int:
   """Canonicalize an axis in [-num_dims, num_dims) to [0, num_dims)."""
   axis = operator.index(axis)
@@ -514,7 +496,7 @@ class HashableWrapper:
 def _original_func(f):
   if isinstance(f, property):
     return cast(property, f).fget
-  elif isinstance(f, cached_property):
+  elif isinstance(f, functools.cached_property):
     return f.func
   return f
 
@@ -539,8 +521,17 @@ def use_cpp_class(cpp_cls):
 
   return wrapper
 
-def use_cpp_method(f):
+
+def use_cpp_method(is_enabled=True):
   """A helper decorator to exclude methods from the set that are forwarded to C++ class"""
-  original_func = _original_func(f)
-  original_func._use_cpp = True
-  return f
+  def decorator(f):
+    if is_enabled:
+      original_func = _original_func(f)
+      original_func._use_cpp = True
+    return f
+
+  if not isinstance(is_enabled, bool):
+    raise TypeError(
+        "Decorator got wrong type: @use_cpp_method(is_enabled: bool=True)"
+    )
+  return decorator
