@@ -210,8 +210,8 @@ same device that issued the outfeed. This device is then responsible for
 sending the required shards to the other devices::
 
   with jax.sharding.Mesh(jax.local_devices()[:2], ["d"]):
-    pjit.pjit(power3, in_axis_resources=(P("d"),),
-              out_axis_resources=(P("d"),))(np.array([3., 4.]))
+    pjit.pjit(power3, in_shardings=(P("d"),),
+              out_shardings=(P("d"),))(np.array([3., 4.]))
 
   # device=TPU:0 what=x,x^2: ( [3., 4.],
   #                            [9., 16.] )
@@ -522,7 +522,7 @@ from jax._src import source_info_util
 from jax._src import util
 from jax._src import lib as jaxlib
 from jax._src.lib import pytree
-from jax._src.lib import xla_bridge as xb
+from jax._src import xla_bridge as xb
 from jax._src.lib import xla_client
 from jax._src.lib import xla_extension
 from jax._src.lib.mlir.dialects import hlo
@@ -1014,7 +1014,7 @@ def _outside_call_impl(*args, **params):
   assert "has_token" not in params
   if _inline_host_callback():
     device_index = params["device_index"]
-    device = api.devices()[device_index]
+    device = xb.devices()[device_index]
     results = _outside_call_run_callback(args, device, send_infeed=False, **params)
     return results
   else:
