@@ -15,24 +15,28 @@
 
 Specific JAX primitive conversion tests are in primitives_test."""
 import collections
+import contextlib
 import os
 from typing import Callable, Dict, Optional, Tuple
 import unittest
 
 from absl import logging
-from absl.testing import absltest
+from absl.testing import absltest, parameterized
 
 import jax
 from jax import ad_checkpoint
 from jax import dtypes
 from jax import lax
 from jax import numpy as jnp
+from jax import sharding
 from jax._src import source_info_util
 from jax._src import test_util as jtu
-import jax._src.lib.xla_bridge
+import jax._src.xla_bridge
 from jax.config import config
 from jax.experimental import jax2tf
 from jax.experimental.jax2tf.tests import tf_test_util
+from jax.experimental.maps import xmap
+from jax.experimental.shard_map import shard_map
 from jax.experimental import pjit
 from jax.interpreters import mlir
 from jax.sharding import PartitionSpec as P
@@ -768,7 +772,11 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     arg = np.array(3.)
     f_tf = jax2tf.convert(jax.grad(remat_f))
     f_tf_hlo = self.TfToHlo(f_tf, arg)
-    self.assertRegex(f_tf_hlo, r"opt-barrier")
+    if jax.config.jax_remat_opt_barrier:
+      self.assertRegex(f_tf_hlo, r"opt-barrier")
+    else:
+      self.assertRegex(f_tf_hlo,
+                       r'transpose/jax2tf_f_/jvp/checkpoint/cond/branch_1_fun/Sin')
 
   def test_remat_free_var(self):
     def f(x):
@@ -1232,7 +1240,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     x = np.arange(np.prod(shape), dtype=np.float32).reshape(shape)
 
     jax_comp = jax.xla_computation(f_while)(x)
-    backend = jax._src.lib.xla_bridge.get_backend()
+    backend = jax._src.xla_bridge.get_backend()
     modules = backend.compile(jax_comp).hlo_modules()
     jax_opt_hlo = modules[0].to_string()
     print(f"JAX OPT HLO = {jax_opt_hlo}")
@@ -1309,9 +1317,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllOperationStartWith(g2, outer_scope)
 
   def test_name_scope_cond(self):
-
     def f(x):
-
       def f_pos(x):
         with jax.named_scope("jax_f_pos"):
           return lax.cond(x < 1., jnp.cos, jnp.sin, x)
@@ -1348,13 +1354,10 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
       self.assertAllOperationStartWith(func.graph, "tf_outer_back")
 
   def test_name_scope_while_loop(self):
-
     def f(x):
       with tf.name_scope("outer_scope"):
-
         def condition(x):
           return jnp.sum(x, keepdims=False) < 100
-
         def body(x):
           return jnp.add(x, 2.0)
 
@@ -1371,18 +1374,114 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
               "tf graph has repeated name issue on when converting lax.while to tf.while."
               f"See op.name = : {op.name}")
 
+  @parameterized.named_parameters(
+      dict(testcase_name=f"{'with_mesh_' if with_mesh else ''}{'nullary_' if nullary else ''}{transform}_pjit_sharding={pjit_sharding}",
+           with_mesh=with_mesh, transform=transform, nullary=nullary, pjit_sharding=pjit_sharding)
+      # The inner transformation to apply to the lowered function
+      for transform in ["base",
+                      "jit",
+                      "pjit", "pjit_in_shardings_None", "pjit_in_shardings_P", "pjit_in_shardings_Sharding",
+                      "shard_map", "xmap", "pmap"]
+      # The sharding to be used for the outer pjit
+      for pjit_sharding in (
+          ["unspecified"] if transform == "pmap" else
+          ["unspecified", "none", "P", "Sharding"])
+      # Whether the function can be nullary
+      for nullary in (
+          [False] if (pjit_sharding != "unspecified") else
+          [True, False]
+      )
+      # Whether we use a "with mesh"
+      for with_mesh in (
+          [True] if (transform not in ["base", "jit", "pjit"] or
+                     pjit_sharding != "unspecified") else
+          [False, True])
+  )
+  def test_cross_platform(self, with_mesh=False, transform="jit", nullary=False, pjit_sharding="unspecified"):
+    # Tests cross-lowering for
+    #  with mesh:
+    #   pjit(transform(func), in_sharding=pjit_sharding)
+    if not config.jax_array:
+      raise unittest.SkipTest("cross_platform test work only with jax.Array")
+    if not config.jax_jit_pjit_api_merge:
+      raise unittest.SkipTest("cross_platform test work only with jax.Array")
+    x = np.ones((4, 6), dtype=np.float32)
+    mesh = sharding.Mesh(jax.devices()[:1], ("a",))
+    # cummax has distinctive lowering for TPU, using a reduce-window op
+    func = lambda x: lax.cummax(x, axis=0, reverse=False)
+    # For shard_map we cannot use cummax :-( because it does not have a replication rule
+    # But we use lax.all_gather which on TPU is lowered with a all-gather op
+    func_shard_map = lambda x: lax.all_gather(x, 'a', axis=1, tiled=True)
+
+    transformed_func = dict(
+        base=func,
+        jit=jax.jit(func),
+        jit_in_shardings_None=jax.jit(func, in_shardings=None),
+        jit_in_shardings_P=jax.jit(func, in_shardings=(P("a"),)),
+        jit_in_shardings_Sharding=jax.jit(func, in_shardings=(sharding.NamedSharding(mesh, P("a")),)),
+        pjit=pjit.pjit(func),
+        pjit_in_shardings_None=pjit.pjit(func, in_shardings=None),
+        pjit_in_shardings_P=pjit.pjit(func, in_shardings=(P("a"),)),
+        pjit_in_shardings_Sharding=pjit.pjit(func, in_shardings=(sharding.NamedSharding(mesh, P("a")),)),
+        shard_map=(
+            shard_map(func_shard_map, mesh, in_specs=(P("a", None),), out_specs=P("a", None))),
+        xmap=xmap(func, in_axes=({0: 'axis'},), out_axes={0: 'axis'}, axis_resources={'axis': 'a'}),
+        pmap=jax.pmap(func, in_axes=0, out_axes=0),
+    )[transform]
+    pjit_transformed_func = dict(
+        unspecified=pjit.pjit(transformed_func),
+        none=pjit.pjit(transformed_func, in_shardings=None),
+        P=pjit.pjit(transformed_func, in_shardings=(P("a"),)),
+        Sharding=pjit.pjit(transformed_func, in_shardings=(sharding.NamedSharding(mesh, P("a")),)),
+    )[pjit_sharding]
+    if pjit_sharding == "unspecified":
+      if transform == "xmap":
+        raise unittest.SkipTest("TODO: pjit(xmap) with unspecified shardings crashes")
+
+    if not nullary:
+      func_to_convert = pjit_transformed_func
+      args = [x]
+    else:
+      func_to_convert = lambda: pjit_transformed_func(jnp.ones(x.shape, dtype=x.dtype))
+      args = []
+
+    if transform == "pmap":
+      if nullary:
+        raise unittest.SkipTest("Cannot lower nested pmap: jit-of-pmap warning")
+      raise unittest.SkipTest("TODO: pmap picks the devices from jax.devices() and will lower for CPU")
+
+    if transform == "xmap":
+      raise unittest.SkipTest("TODO: xmap does not pick up the overriden mesh and will lower for CPU")
+
+    f_tf = jax2tf.convert(func_to_convert,
+                          experimental_native_lowering=True,
+                          experimental_native_lowering_platforms=('tpu',))
+    f_tf = tf.function(f_tf, jit_compile=True, autograph=False)
+    with contextlib.ExitStack() as stack:
+      if with_mesh:
+        stack.enter_context(mesh)
+      # Run the JAX native version, to check it works, and to fill caches.
+      _ = func_to_convert(*args)
+      tf_hlo = f_tf.experimental_get_compiler_ir(*args)(stage="hlo")
+
+    if transform == "shard_map":
+      self.assertIn("all-gather(f32[4,6]", tf_hlo)
+    else:
+      self.assertIn("reduce-window(f32[4,6]", tf_hlo)
+
+
 def get_serialized_computation(
     f_jax: Callable,
     *args,
     abstracted_axes: Optional[Tuple[Dict[int, str]]] = None,
     use_pjit: bool = False,
-    in_axis_resources = None,
-    out_axis_resources = None) -> str:
+    in_shardings = None,
+    out_shardings = None) -> str:
   if use_pjit:
     assert not abstracted_axes
-    lowered = pjit.pjit(f_jax,
-                   in_axis_resources=in_axis_resources,
-                   out_axis_resources=out_axis_resources).lower(*args)
+    lowered = pjit.pjit(
+        f_jax, in_shardings=in_shardings, out_shardings=out_shardings
+  ).lower(*args)
   else:
     lowered = jax.jit(f_jax, abstracted_axes=abstracted_axes).lower(*args)
   stablehlo_module_text = mlir.module_to_string(lowered._lowering.stablehlo())
@@ -1485,15 +1584,16 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     out_axis_resources = None
     res_jax = pjit.pjit(
         func_jax,
-        in_axis_resources=in_axis_resources,
-        out_axis_resources=out_axis_resources)(x, x)
+        in_shardings=in_axis_resources,
+        out_shardings=out_axis_resources,
+    )(x, x)
     module = get_serialized_computation(
         func_jax,
         x,
         x,
         use_pjit=True,
-        in_axis_resources=in_axis_resources,
-        out_axis_resources=out_axis_resources)
+        in_shardings=in_axis_resources,
+        out_shardings=out_axis_resources)
 
     def f_tf(x_tf, y_tf):
       return tfxla.call_module([x_tf, y_tf],
@@ -1507,4 +1607,7 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
 
 
 if __name__ == "__main__":
+  # TODO: Remove once tensorflow is 2.10.0 everywhere.
+  if not hasattr(tfxla, "optimization_barrier"):
+    jax.config.update("jax_remat_opt_barrier", False)
   absltest.main(testLoader=jtu.JaxTestLoader())
