@@ -127,27 +127,11 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
         "top_k_sort_inf_nan_inshape=float32[5]_k=5" in harness.fullname):
       raise unittest.SkipTest("Unexplained failure, but in old no_jax_array")
 
-    if (config.jax2tf_default_experimental_native_lowering and
-        "cholesky" in harness.fullname):
-      raise unittest.SkipTest("b/269386856: cholesky failures")
-
-    if (config.jax2tf_default_experimental_native_lowering and
-        device in ["cpu", "gpu"] and
-        "igammac" in harness.fullname):
-      raise unittest.SkipTest("b/269401509: igammac failures")
-
-    if (config.jax2tf_default_experimental_native_lowering and
-        device == "tpu" and
-        harness.dtype == jnp.bfloat16 and
-        "eigh" in harness.fullname):
-      raise unittest.SkipTest("b/269388842: eigh failures on TPU for bfloat16")
-
     if (config.jax2tf_default_experimental_native_lowering and
         device == "gpu" and
         "lu" in harness.fullname):
       raise unittest.SkipTest("b/269388847: lu failures on GPU")
 
-
     associative_scan_reductions = harness.params.get("associative_scan_reductions", False)
     try:
       with jax.jax2tf_associative_scan_reductions(associative_scan_reductions):
@@ -309,10 +293,13 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
     y = np.int32(3)
     self.ConvertAndCompare(jnp.floor_divide, x, y)
     expected = jnp.floor_divide(x, y)
-    # Try it with TF 1 as well (#5831)
-    with tf.compat.v1.Session() as sess:
-      tf1_res = sess.run(jax2tf.convert(jnp.floor_divide)(x, y))
-      self.assertAllClose(expected, tf1_res)
+    if not config.jax2tf_default_experimental_native_lowering:
+      # With native lowering TF1 seems to want to run the converted code
+      # on the CPU even when the default backend is the TPU.
+      # Try it with TF 1 as well (#5831)
+      with tf.compat.v1.Session() as sess:
+        tf1_res = sess.run(jax2tf.convert(jnp.floor_divide)(x, y))
+        self.assertAllClose(expected, tf1_res)
 
   def test_boolean_gather(self):
     values = np.array([[True, True], [False, True], [False, False]],
