@@ -31,7 +31,7 @@ from jax.interpreters import xla
 from jax._src import pjit as pjit_lib
 from jax.experimental.pjit import pjit, FROM_GDA
 from jax.sharding import PartitionSpec as P
-from jax.experimental.global_device_array import GlobalDeviceArray
+from jax._src.global_device_array import GlobalDeviceArray
 from jax._src import distributed
 from jax._src import config as config_internal
 import numpy as np
@@ -96,9 +96,9 @@ def _identity_fn(x):
 
 def _handle_array_process_allgather(inp, tiled):
   if isinstance(inp, array.ArrayImpl) and not inp.is_fully_addressable:
-    reps = sharding.OpShardingSharding(inp.sharding._device_assignment,
+    reps = sharding.GSPMDSharding(inp.sharding._device_assignment,
                                        sharding._get_replicated_op_sharding())
-    out = pjit(_identity_fn, out_axis_resources=reps)(inp)
+    out = pjit(_identity_fn, out_shardings=reps)(inp)
   else:
     # All inputs here will be fully addressable.
     if jax.process_count() == 1:
@@ -122,7 +122,7 @@ def _handle_array_process_allgather(inp, tiled):
     global_arr = array.make_array_from_single_device_arrays(
         global_aval.shape, s, bufs)
     with global_mesh:
-      out = pjit(_identity_fn, out_axis_resources=None)(global_arr)
+      out = pjit(_identity_fn, out_shardings=None)(global_arr)
 
   return np.asarray(out.addressable_data(0))
 
@@ -168,8 +168,9 @@ def process_allgather(in_tree: PyTreeDef, tiled: bool = False) -> PyTreeDef:
           inp = np.expand_dims(inp, axis=0)
 
       with global_mesh:
-        out = pjit(_identity_fn, in_axis_resources=in_axis_resources,
-                   out_axis_resources=None)(inp)
+        out = pjit(
+            _identity_fn, in_shardings=in_axis_resources, out_shardings=None
+        )(inp)
       return np.asarray(out.addressable_data(0))
 
   if jax.config.jax_array:
