@@ -17,6 +17,7 @@ import enum
 from functools import partial, lru_cache
 import inspect
 import itertools as it
+import math
 import operator as op
 from typing import (Any, Callable, Dict, Hashable, List, Optional, Sequence,
                     Set, Tuple, TypeVar, Union, Protocol)
@@ -37,7 +38,7 @@ from jax._src import util
 from jax._src.core import Tracer
 from jax._src.lax import (lax, parallel as lax_parallel, slicing,
                           windowed_reductions, fft, linalg)
-from jax._src.util import (prod, HashableFunction, unzip2, as_hashable_function,
+from jax._src.util import (HashableFunction, unzip2, as_hashable_function,
                            memoize, partition_list, merge_lists)
 from jax.api_util import flatten_fun_nokwargs, shaped_abstractify
 from jax.interpreters import batching
@@ -150,7 +151,7 @@ def _check_specs_vs_args(
     msg = _spec_rank_error(SpecErrorType.input, f, in_tree, in_specs, fail)
     raise ValueError(msg)
   in_names_flat = tuple(map(_canonicalize_spec, in_specs_flat))
-  fail = [a if any(a.shape[d] % prod(mesh.shape[n] for n in ns)
+  fail = [a if any(a.shape[d] % math.prod(mesh.shape[n] for n in ns)
                    for d, ns in names.items()) else no_fail
           for a, names in zip(in_avals, in_names_flat)]
   if any(f is not no_fail for f in fail):
@@ -201,10 +202,10 @@ def _spec_divisibility_error(
                f"parameter '{list(ba.arguments.keys())[arg_key.key]}',")
     names = _canonicalize_spec(spec)
     for d, ns in names.items():
-      if aval.shape[d] % prod(mesh.shape[n] for n in ns):
+      if aval.shape[d] % math.prod(mesh.shape[n] for n in ns):
         axis = f"axes {ns}" if len(ns) > 1 else f"axis '{ns[0]}'"
         total = 'total ' if len(ns) > 1 else ''
-        sz = prod(mesh.shape[n] for n in ns)
+        sz = math.prod(mesh.shape[n] for n in ns)
         msgs.append(
             f"args{fail_key.pprint()} of shape {aval.str_short()}{extra} "
             f"corresponds to in_specs{spec_key.pprint()} of value {spec}, "
@@ -313,7 +314,6 @@ class ShardMapPrimitive(core.Primitive):
     return map(core.full_lower, core.apply_todos(todos, outs))
 
   def get_bind_params(self, params):
-    """Goes from jaxpr form to python traceable form."""
     new_params = dict(params)
     jaxpr = new_params.pop('jaxpr')
     subfun = lu.hashable_partial(lu.wrap_init(core.eval_jaxpr), jaxpr, ())
@@ -383,7 +383,7 @@ pe.DynamicJaxprTrace.process_shard_map = _shard_map_staging
 def _shard_aval(mesh: Mesh, names: AxisNames, aval: core.AbstractValue
                 ) -> core.AbstractValue:
   if isinstance(aval, core.ShapedArray):
-    return aval.update(tuple(sz // prod(mesh.shape[n] for n in names.get(i, ()))
+    return aval.update(tuple(sz // math.prod(mesh.shape[n] for n in names.get(i, ()))
                              for i, sz in enumerate(aval.shape)))
   else:
     raise NotImplementedError  # TODO(mattjj): add table with handlers
@@ -391,7 +391,7 @@ def _shard_aval(mesh: Mesh, names: AxisNames, aval: core.AbstractValue
 def _unshard_aval(mesh: Mesh, names: AxisNames, aval: core.AbstractValue
                  ) -> core.AbstractValue:
   if isinstance(aval, core.ShapedArray):
-    return aval.update(tuple(sz * prod(mesh.shape[n] for n in names.get(i, ()))
+    return aval.update(tuple(sz * math.prod(mesh.shape[n] for n in names.get(i, ()))
                              for i, sz in enumerate(aval.shape)),
                        named_shape={k: v for k, v in aval.named_shape.items()
                                     if k not in mesh.shape})
@@ -522,7 +522,7 @@ def _unmatch(mesh, src_tup, x):
 
 def _check_names(names: Sequence[AxisNames], avals: Sequence[core.ShapedArray]
                  ) -> None:
-  fail = [a if not max(n, default=0) < a.ndim else no_fail
+  fail = [a if n and not max(n) < a.ndim else no_fail
           for n, a in zip(names, avals)]
   if any(f is not no_fail for f in fail): raise _SpecError(fail)
 class _SpecError(Exception): pass
@@ -654,7 +654,6 @@ for o in it.chain(lax.__dict__.values(), slicing.__dict__.values(),
                   custom_derivatives.__dict__.values()):
   if isinstance(o, core.Primitive): register_standard(o)
 
-register_standard(xla.xla_call_p)
 register_standard(lax_parallel.ppermute_p)  # doesn't change replication
 
 @register_rule(lax_parallel.psum_p)
@@ -693,6 +692,10 @@ def _axis_index_rule(mesh, *, axis_name):
 def _pjit_rule(mesh, *in_rep, jaxpr, **kwargs):
   return _output_rep(mesh, jaxpr.jaxpr, in_rep)
 
+@register_rule(xla.xla_call_p)
+def _jit_rule(mesh, *in_rep, jaxpr, **kwargs):
+  return _output_rep(mesh, jaxpr, in_rep)
+
 # Batching
 
 def _shard_map_batch(
@@ -754,7 +757,7 @@ def _shard_map_jvp(trace, shard_map_p, f, tracers, mesh, in_names,
   result = shard_map_p.bind(f_jvp, *args, **params)
   primal_out, tangent_out = tree_unflatten(out_tree(), result)
   tangent_out = [ad.Zero(core.get_aval(p).at_least_vspace()) if t is None else t
-              for p, t in zip(primal_out, tangent_out)]
+                 for p, t in zip(primal_out, tangent_out)]
   return [ad.JVPTracer(trace, p, t) for p, t in zip(primal_out, tangent_out)]
 ad.JVPTrace.process_shard_map = _shard_map_jvp
 
@@ -783,7 +786,6 @@ def _shard_map_partial_eval(trace, shard_map_p, f, tracers, mesh, in_names,
   f = _promote_scalar_residuals(f)
   f_known, aux = pe.partial_eval_wrapper_nounits(
       f, (*in_knowns,), (*in_avals_sharded,))
-  unk_in_names, known_in_names = pe.partition_list(in_knowns, in_names)
 
   @as_hashable_function(closure=out_names_thunk)
   def known_out_names():
@@ -879,7 +881,7 @@ def _shard_map_transpose(out_cts, *args, jaxpr, mesh, in_names, out_names,
                          check_rep):
   mb_div = lambda x, y: x / y if y != 1 else x
   out_cts = [ad.Zero(_shard_aval(mesh, ns, x.aval)) if type(x) is ad.Zero
-             else mb_div(x, prod(map(mesh.shape.get, _unmentioned(mesh, ns))))
+             else mb_div(x, math.prod(map(mesh.shape.get, _unmentioned(mesh, ns))))
              for ns, x in zip(out_names, out_cts)]
   args = [x if type(x) is not ad.UndefinedPrimal else
           ad.UndefinedPrimal(_shard_aval(mesh, ns, x.aval))
