@@ -13,10 +13,11 @@
 # limitations under the License.
 
 """CSR (compressed sparse row) matrix object and associated primitives."""
+from __future__ import annotations
 
 from functools import partial
 import operator
-from typing import Tuple
+from typing import Optional, Tuple
 import warnings
 
 import numpy as np
@@ -33,9 +34,13 @@ from jax._src.interpreters import ad
 from jax._src.lax.lax import _const
 from jax._src.lib import gpu_sparse
 from jax._src.numpy.lax_numpy import _promote_dtypes
+from jax._src.typing import Array, ArrayLike, DTypeLike
 import jax.numpy as jnp
 
 
+Shape = Tuple[int, ...]
+
+
 @tree_util.register_pytree_node_class
 class CSR(JAXSparse):
   """Experimental CSR matrix implemented in JAX.
@@ -43,6 +48,10 @@ class CSR(JAXSparse):
   Note: this class has minimal compatibility with JAX transforms such as
   grad and autodiff, and offers very little functionality. In general you
   should prefer :class:`jax.experimental.sparse.BCOO`.
+
+  Additionally, there are known failures in the case that `nse` is larger
+  than the true number of nonzeros in the represented matrix. This situation
+  is better handled in BCOO.
   """
   data: jax.Array
   indices: jax.Array
@@ -50,6 +59,7 @@ class CSR(JAXSparse):
   shape: Tuple[int, int]
   nse = property(lambda self: self.data.size)
   dtype = property(lambda self: self.data.dtype)
+  _bufs = property(lambda self: (self.data, self.indices, self.indptr))
 
   def __init__(self, args, *, shape):
     self.data, self.indices, self.indptr = map(jnp.asarray, args)
@@ -59,7 +69,7 @@ class CSR(JAXSparse):
   def fromdense(cls, mat, *, nse=None, index_dtype=np.int32):
     if nse is None:
       nse = (mat != 0).sum()
-    return cls(csr_fromdense(mat, nse=nse, index_dtype=index_dtype), shape=mat.shape)
+    return csr_fromdense(mat, nse=nse, index_dtype=index_dtype)
 
   @classmethod
   def _empty(cls, shape, *, dtype=None, index_dtype='int32'):
@@ -96,7 +106,7 @@ class CSR(JAXSparse):
     return cls((data, indices, indptr), shape=(N, M))
 
   def todense(self):
-    return csr_todense(self.data, self.indices, self.indptr, shape=self.shape)
+    return csr_todense(self)
 
   def transpose(self, axes=None):
     assert axes is None
@@ -108,9 +118,9 @@ class CSR(JAXSparse):
     other = jnp.asarray(other)
     data, other = _promote_dtypes(self.data, other)
     if other.ndim == 1:
-      return csr_matvec(data, self.indices, self.indptr, other, shape=self.shape)
+      return _csr_matvec(data, self.indices, self.indptr, other, shape=self.shape)
     elif other.ndim == 2:
-      return csr_matmat(data, self.indices, self.indptr, other, shape=self.shape)
+      return _csr_matmat(data, self.indices, self.indptr, other, shape=self.shape)
     else:
       raise NotImplementedError(f"matmul with object of shape {other.shape}")
 
@@ -145,7 +155,7 @@ class CSC(JAXSparse):
   def fromdense(cls, mat, *, nse=None, index_dtype=np.int32):
     if nse is None:
       nse = (mat != 0).sum()
-    return cls(csr_fromdense(mat.T, nse=nse, index_dtype=index_dtype), shape=mat.shape)
+    return csr_fromdense(mat.T, nse=nse, index_dtype=index_dtype).T
 
   @classmethod
   def _empty(cls, shape, *, dtype=None, index_dtype='int32'):
@@ -163,7 +173,7 @@ class CSC(JAXSparse):
     return CSR._eye(M, N, -k, dtype=dtype, index_dtype=index_dtype).T
 
   def todense(self):
-    return csr_todense(self.data, self.indices, self.indptr, shape=self.shape[::-1]).T
+    return csr_todense(self.T).T
 
   def transpose(self, axes=None):
     assert axes is None
@@ -175,9 +185,11 @@ class CSC(JAXSparse):
     other = jnp.asarray(other)
     data, other = _promote_dtypes(self.data, other)
     if other.ndim == 1:
-      return csr_matvec(data, self.indices, self.indptr, other, shape=self.shape[::-1], transpose=True)
+      return _csr_matvec(data, self.indices, self.indptr, other,
+                         shape=self.shape[::-1], transpose=True)
     elif other.ndim == 2:
-      return csr_matmat(data, self.indices, self.indptr, other, shape=self.shape[::-1], transpose=True)
+      return _csr_matmat(data, self.indices, self.indptr, other,
+                         shape=self.shape[::-1], transpose=True)
     else:
       raise NotImplementedError(f"matmul with object of shape {other.shape}")
 
@@ -199,7 +211,17 @@ class CSC(JAXSparse):
 
 csr_todense_p = core.Primitive('csr_todense')
 
-def csr_todense(data, indices, indptr, *, shape):
+def csr_todense(mat: CSR) -> Array:
+  """Convert a CSR-format sparse matrix to a dense matrix.
+
+  Args:
+    mat : CSR matrix
+  Returns:
+    mat_dense: dense version of ``mat``
+  """
+  return _csr_todense(mat.data, mat.indices, mat.indptr, shape=mat.shape)
+
+def _csr_todense(data: Array, indices: Array, indptr: Array, *, shape: Shape) -> Array:
   """Convert CSR-format sparse matrix to a dense matrix.
 
   Args:
@@ -242,7 +264,7 @@ def _csr_todense_gpu_lowering(csr_todense_hlo, ctx, data, indices, indptr, *,
 
 
 def _csr_todense_jvp(data_dot, data, indices, indptr, *, shape):
-  return csr_todense(data_dot, indices, indptr, shape=shape)
+  return _csr_todense(data_dot, indices, indptr, shape=shape)
 
 def _csr_todense_transpose(ct, data, indices, indptr, *, shape):
   # Note: we assume that transpose has the same sparsity pattern.
@@ -276,7 +298,24 @@ if gpu_sparse.rocm_is_supported:
 csr_fromdense_p = core.Primitive('csr_fromdense')
 csr_fromdense_p.multiple_results = True
 
-def csr_fromdense(mat, *, nse, index_dtype=np.int32):
+def csr_fromdense(mat: Array, *, nse: Optional[int] = None, index_dtype: DTypeLike = np.int32) -> CSR:
+  """Create a CSR-format sparse matrix from a dense matrix.
+
+  Args:
+    mat : array to be converted to CSR.
+    nse : number of specified entries in ``mat``. If not specified,
+      it will be computed from the input matrix.
+    index_dtype : dtype of sparse indices
+
+  Returns:
+    mat_coo : CSR representation of the matrix.
+  """
+  if nse is None:
+    nse = int((mat != 0).sum())
+  nse_int = core.concrete_or_error(operator.index, nse, "coo_fromdense nse argument")
+  return CSR(_csr_fromdense(mat, nse=nse_int, index_dtype=index_dtype), shape=mat.shape)
+
+def _csr_fromdense(mat: Array, *, nse: int, index_dtype: DTypeLike = np.int32) -> Tuple[Array, Array, Array]:
   """Create CSR-format sparse matrix from a dense matrix.
 
   Args:
@@ -336,7 +375,7 @@ def _csr_fromdense_jvp(primals, tangents, *, nse, index_dtype):
   M, = primals
   Mdot, = tangents
 
-  primals_out = csr_fromdense(M, nse=nse, index_dtype=index_dtype)
+  primals_out = _csr_fromdense(M, nse=nse, index_dtype=index_dtype)
   data, indices, indptr = primals_out
 
   if type(Mdot) is ad.Zero:
@@ -355,7 +394,7 @@ def _csr_fromdense_transpose(ct, M, *, nse, index_dtype):
   if isinstance(indices, ad.Zero) or isinstance(indptr, ad.Zero):
     raise ValueError("Cannot transpose with respect to sparse indices")
   assert ad.is_undefined_primal(M)
-  return csr_todense(data, indices, indptr, shape=M.aval.shape)
+  return _csr_todense(data, indices, indptr, shape=M.aval.shape)
 
 ad.primitive_jvps[csr_fromdense_p] = _csr_fromdense_jvp
 ad.primitive_transposes[csr_fromdense_p] = _csr_fromdense_transpose
@@ -376,7 +415,24 @@ if gpu_sparse.rocm_is_supported:
 
 csr_matvec_p = core.Primitive('csr_matvec')
 
-def csr_matvec(data, indices, indptr, v, *, shape, transpose=False):
+def csr_matvec(mat: CSR, v: Array, transpose: bool = False) -> Array:
+  """Product of CSR sparse matrix and a dense vector.
+
+  Args:
+    mat : CSR matrix
+    v : one-dimensional array of size ``(shape[0] if transpose else shape[1],)`` and
+      dtype ``mat.dtype``
+    transpose : boolean specifying whether to transpose the sparse matrix
+      before computing.
+
+  Returns:
+    y : array of shape ``(mat.shape[1] if transpose else mat.shape[0],)`` representing
+      the matrix vector product.
+  """
+  data, indices, indptr = mat._bufs
+  return _csr_matvec(data, indices, indptr, v, shape=mat.shape, transpose=transpose)
+
+def _csr_matvec(data, indices, indptr, v, *, shape, transpose=False):
   """Product of CSR sparse matrix and a dense vector.
 
   Args:
@@ -428,17 +484,17 @@ def _csr_matvec_gpu_lowering(csr_matvec_hlo, ctx, data, indices, indptr, v, *,
 
 
 def _csr_matvec_jvp_mat(data_dot, data, indices, indptr, v, *, shape, transpose):
-  return csr_matvec(data_dot, indices, indptr, v, shape=shape, transpose=transpose)
+  return _csr_matvec(data_dot, indices, indptr, v, shape=shape, transpose=transpose)
 
 def _csr_matvec_jvp_vec(v_dot, data, indices, indptr, v, *, shape, transpose):
-  return csr_matvec(data, indices, indptr, v_dot, shape=shape, transpose=transpose)
+  return _csr_matvec(data, indices, indptr, v_dot, shape=shape, transpose=transpose)
 
 def _csr_matvec_transpose(ct, data, indices, indptr, v, *, shape, transpose):
   assert not ad.is_undefined_primal(indices)
   assert not ad.is_undefined_primal(indptr)
 
   if ad.is_undefined_primal(v):
-    return data, indices, indptr, csr_matvec(data, indices, indptr, ct, shape=shape, transpose=not transpose)
+    return data, indices, indptr, _csr_matvec(data, indices, indptr, ct, shape=shape, transpose=not transpose)
   else:
     v = jnp.asarray(v)
     # The following lines do this, but more efficiently.
@@ -467,7 +523,25 @@ if gpu_sparse.rocm_is_supported:
 
 csr_matmat_p = core.Primitive('csr_matmat')
 
-def csr_matmat(data, indices, indptr, B, *, shape, transpose=False):
+def csr_matmat(mat: CSR, B: Array, *, transpose: bool = False) -> Array:
+  """Product of CSR sparse matrix and a dense matrix.
+
+  Args:
+    mat : CSR matrix
+    B : array of shape ``(mat.shape[0] if transpose else mat.shape[1], cols)`` and
+      dtype ``mat.dtype``
+    transpose : boolean specifying whether to transpose the sparse matrix
+      before computing.
+
+  Returns:
+    C : array of shape ``(mat.shape[1] if transpose else mat.shape[0], cols)``
+      representing the matrix vector product.
+  """
+  data, indices, indptr = mat._bufs
+  return _csr_matmat(data, indices, indptr, B, shape=mat.shape, transpose=transpose)
+
+def _csr_matmat(data: Array, indices: Array, indptr: Array, B: Array,
+                *, shape: Shape, transpose: bool = False) -> Array:
   """Product of CSR sparse matrix and a dense matrix.
 
   Args:
@@ -521,17 +595,17 @@ def _csr_matmat_gpu_lowering(csr_matmat_hlo, ctx, data, indices, indptr, B, *,
 
 
 def _csr_matmat_jvp_left(data_dot, data, indices, indptr, B, *, shape, transpose):
-  return csr_matmat(data_dot, indices, indptr, B, shape=shape, transpose=transpose)
+  return _csr_matmat(data_dot, indices, indptr, B, shape=shape, transpose=transpose)
 
 def _csr_matmat_jvp_right(B_dot, data, indices, indptr, B, *, shape, transpose):
-  return csr_matmat(data, indices, indptr, B_dot, shape=shape, transpose=transpose)
+  return _csr_matmat(data, indices, indptr, B_dot, shape=shape, transpose=transpose)
 
 def _csr_matmat_transpose(ct, data, indices, indptr, B, *, shape, transpose):
   assert not ad.is_undefined_primal(indices)
   assert not ad.is_undefined_primal(indptr)
 
   if ad.is_undefined_primal(B):
-    return data, indices, indptr, csr_matmat(data, indices, indptr, ct, shape=shape, transpose=not transpose)
+    return data, indices, indptr, _csr_matmat(data, indices, indptr, ct, shape=shape, transpose=not transpose)
   else:
     B = jnp.asarray(B)
     row, col = _csr_to_coo(indices, indptr)
