@@ -95,7 +95,6 @@ from jax._src.interpreters.pxla import (
   mesh_sharding_specs as mesh_sharding_specs,
   multi_host_supported_collectives as multi_host_supported_collectives,
   new_mesh_sharding_specs as new_mesh_sharding_specs,
-  new_name_stack as new_name_stack,
   op_sharding_to_indices as op_sharding_to_indices,
   parallel_callable as parallel_callable,
   partitioned_sharding_spec as partitioned_sharding_spec,
@@ -105,6 +104,7 @@ from jax._src.interpreters.pxla import (
   sda_array_result_handler as sda_array_result_handler,
   shard_arg_handlers as shard_arg_handlers,
   shard_args as shard_args,
+  shard_arg as shard_arg,
   shard_aval as shard_aval,
   shard_aval_handlers as shard_aval_handlers,
   shard_to_full_p as shard_to_full_p,
